# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Tue Apr 28 11:00:55 2015

@author: Vidar Tonaas Fauske
"""

from hyperspyui.plugins.plugin import Plugin

from qtpy import QtCore, QtWidgets
from qtpy.QtWidgets import (QLabel, QPushButton, QToolButton, QVBoxLayout,
                            QHBoxLayout, QWidget)

import collections

from hyperspyui.util import win2sig
from hyperspyui.widgets.extendedqwidgets import FigureWidget
from hyperspy.axes import DataAxis
from hyperspy.signal import BaseSignal


def tr(text):
    return QtCore.QCoreApplication.translate("AxesOrderPlugin", text)


class AxesOrderPlugin(Plugin):
    name = "Axes order widget"

    def create_widgets(self):
        w = AxesOrderWidget(self.ui, self.ui)
        w.plugin = self
        self.add_widget(w)
        w.hide()  # Initial state hidden

    def flip_axes(self, axes, signal=None):
        if signal is None:
            signal = self.ui.get_selected_signal()
        self.record_code("<p>.flip_axes(axes=%s)" % axes)
        if not isinstance(axes, collections.abc.Iterable):
            axes = (axes,)
        replot = False
        for axis in axes:
            if not isinstance(axis, DataAxis):
                axis = signal.axes_manager[axis]
            ax = axis.index_in_array
            slices = (slice(None),) * ax + (slice(None, None, -1), Ellipsis)
            signal.data = signal.data[slices]
            replot |= axis.navigate
        if replot:
            signal.plot()
        else:
            signal.update_plot()

    def rollaxis(self, axis, pos, signal=None):
        # Get signal
        if signal is None:
            signal = self.ui.get_selected_wrapper()
        elif isinstance(signal, BaseSignal):
            signal = self.ui.lut_signalwrapper[signal]
        am = signal.signal.axes_manager
        # Get DataAxis
        if not isinstance(axis, DataAxis):
            axis = am[axis]
        # Reorder
        old_idx = am._axes.index(axis) + 3j
        new_s = signal.signal.rollaxis(old_idx, pos)
        signal.switch_signal(new_s)
        self.record_code("<p>.rollaxis(%s, %s)" % (str(old_idx), str(pos)))
        signal.replot()


class AxesOrderWidget(FigureWidget):

    def __init__(self, ui, parent=None):
        super(AxesOrderWidget, self).__init__(ui, parent)
        self.setWindowTitle(tr("Axes order"))
        self.signal = None
        self.create_controls()
        self._resize_controls()

    def _on_figure_change(self, win):
        super(AxesOrderWidget, self)._on_figure_change(win)
        self.signal = win2sig(win)
        self._update_controls_from_signal(self.signal)

    def _update_controls_from_signal(self, signal):
        if signal is None:
            self.setEnabled(False)
            return
        self.setEnabled(True)
        self.lst_nav.setUpdatesEnabled(False)
        self.lst_sig.setUpdatesEnabled(False)
        self._updating = True

        self.lst_nav.clear()
        self.lst_sig.clear()

        for ax in signal.signal.axes_manager._get_axes_in_natural_order():
            rep = '%s axis, size: %i' % (ax._get_name(), ax.size)
            p = self.lst_nav if ax.navigate else self.lst_sig
            i = QtWidgets.QListWidgetItem(rep)
            i.setData(QtCore.Qt.UserRole, ax)
            p.addItem(i)

        self._updating = False
        self.lst_nav.setUpdatesEnabled(True)
        self.lst_sig.setUpdatesEnabled(True)
        self._resize_controls()

    def _resize_controls(self):
        self.lst_nav.setUpdatesEnabled(False)
        self.lst_sig.setUpdatesEnabled(False)
        max_nav = max(self.lst_nav.sizeHint().height(), 20)
        max_sig = max(self.lst_sig.sizeHint().height(), 20)
        self.lst_nav.setFixedHeight(max_nav)
        self.lst_sig.setFixedHeight(max_sig)
        self.setFixedHeight(max_nav + max_sig + 150)
        self.lst_nav.setUpdatesEnabled(True)
        self.lst_sig.setUpdatesEnabled(True)

    def _move_item(self, item):
        ax = item.data(QtCore.Qt.UserRole)
        ax.navigate = not ax.navigate
        self.ui.record_code((
            "ax = ui.get_selected_signal().axes_manager[%s]\n"
            "ax.navigate = not ax.navigate\n"
            "ui.get_selected_wrapper().replot()") % (ax.index_in_array + 3j))
        self.signal.replot()

    def _list_move(self, item, dst_row, dst):
        """Called when drag and drop moved interal in list.
        """
        if self._updating:
            return
        ax = item.data(QtCore.Qt.UserRole)
        space = 1j if ax.navigate else 2j
        new_idx = dst_row + space
        self.plugin.rollaxis(ax, new_idx)

    def _list_insert(self, src_row, dst_row, dst):
        """Called when drag and drop moved between lists.
        """
        if self._updating:
            return
        # Switch space
        am = self.signal.signal.axes_manager
        old_space = 1j if dst is self.lst_sig else 2j
        old_idx = src_row + old_space
        ax = am[old_idx]
        dst.item(dst_row).setData(QtCore.Qt.UserRole, ax)
        ax.navigate = not ax.navigate
        self.ui.record_code((
            "ax = ui.get_selected_signal().axes_manager[%s]\n"
            "ax.navigate = not ax.navigate\n") % (ax.index_in_array + 3j))
        space = 1j if ax.navigate else 2j
        new_idx = dst_row + space
        self.plugin.rollaxis(ax, new_idx)

    def _move_down(self):
        if self.lst_nav.count() == 0:
            # nothing to transfer...
            return

        i = self.lst_nav.takeItem(self.lst_nav.currentRow())
        self.lst_sig.addItem(i)
        self._move_item(i)

    def _move_up(self):
        if self.lst_sig.count() == 0:
            # nothing to transfer...
            return
        # TODO: Track "active" list (only select in last?), and move int/ext as
        # needed
        i = self.lst_sig.takeItem(self.lst_sig.currentRow())
        self.lst_nav.addItem(i)
        self._move_item(i)

    def _flip_clicked(self):
        # Get selected axes
        axes = []
        for lst in (self.lst_nav, self.lst_sig):
            items = lst.selectedItems()
            axes.extend([i.data(QtCore.Qt.UserRole) for i in items])
        self.plugin.flip_axes(axes)

    def create_controls(self):
        self.lbl_nav = QLabel(tr("Navigate"), self)
        self.lst_nav = AxesListWidget(self)
        self.btn_up = QToolButton(self)
        self.btn_up.setArrowType(QtCore.Qt.UpArrow)
        self.btn_down = QToolButton(self)
        self.btn_down.setArrowType(QtCore.Qt.DownArrow)
        self.lbl_sig = QLabel(tr("Signal"), self)
        self.lst_sig = AxesListWidget(self)

        sp = self.lst_sig.sizePolicy()
        sp.setVerticalPolicy(QtWidgets.QSizePolicy.Fixed)
        self.lst_sig.setSizePolicy(sp)
        sp = self.lst_nav.sizePolicy()
        sp.setVerticalPolicy(QtWidgets.QSizePolicy.Fixed)
        self.lst_nav.setSizePolicy(sp)

        self.btn_down.clicked.connect(self._move_down)
        self.btn_up.clicked.connect(self._move_up)
        self.lst_nav.inserted.connect(self._list_insert)
        self.lst_sig.inserted.connect(self._list_insert)
        self.lst_nav.moved.connect(self._list_move)
        self.lst_sig.moved.connect(self._list_move)

        self.btn_flip = QPushButton(tr("Reverse axes"))
        self.btn_flip.clicked.connect(self._flip_clicked)

        vbox = QVBoxLayout()
        vbox.addWidget(self.lbl_nav)
        vbox.addWidget(self.lst_nav)
        hbox = QHBoxLayout()
        hbox.addWidget(self.btn_up)
        hbox.addWidget(self.btn_down)
        vbox.addLayout(hbox)
        vbox.addWidget(self.lbl_sig)
        vbox.addWidget(self.lst_sig)
        vbox.addWidget(self.btn_flip)

        w = QWidget()
        w.setLayout(vbox)
        self.setWidget(w)


class AxesListWidget(QtWidgets.QListWidget):
    inserted = QtCore.Signal(int, int, QtWidgets.QListWidget)
    moved = QtCore.Signal(QtWidgets.QListWidgetItem, int, QtWidgets.QListWidget)

    last_drop = None

    def __init__(self, type, parent=None):
        super(AxesListWidget, self).__init__(parent)
        self.setIconSize(QtCore.QSize(124, 124))
        self.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.setAcceptDrops(True)
        m = self.model()
        m.rowsInserted.connect(self._on_rows_inserted)
        m.rowsMoved.connect(self._on_rows_moved)

    def minimumSizeHint(self):
        s = QtCore.QSize()
        shfr = self.sizeHintForRow(0)
        if shfr < 0:
            shfr = 12
        h = shfr * self.count() + 2 * self.frameWidth()
        s.setHeight(h)
        s.setWidth(super(AxesListWidget, self).minimumSizeHint().width())
        return s

    def sizeHint(self):
        s = self.minimumSizeHint()
        s.setWidth(super(AxesListWidget, self).sizeHint().width())
        return s

    def _on_rows_inserted(self, parent, begin, end):
        for new_idx in range(begin, end+1):
            if AxesListWidget.last_drop:
                old_idx = AxesListWidget.last_drop.pop(0)
                self.inserted.emit(old_idx, new_idx, self)

    def _on_rows_moved(self, sourceParent, sourceStart, sourceEnd,
                       destinationParent, destinationRow):
        N = sourceEnd - sourceStart + 1
        for i in range(N):
            idx = destinationRow + i
            if destinationRow > sourceStart:
                idx -= N
            item = self.item(idx)
            self.moved.emit(item, idx, self)

    def decodeMimeData(self, data):
        result = {}
        stream = QtCore.QDataStream(data)
        while not stream.atEnd():
            row = stream.readInt32()
            stream.readInt32()                      # Column; not used
            item = result.setdefault(row, {})
            for role in range(stream.readInt32()):
                key = QtCore.Qt.ItemDataRole(stream.readInt32())
                item[key] = stream.readQVariant()
        return result

    def dropEvent(self, event):
        event.setDropAction(QtCore.Qt.MoveAction)
        m = event.mimeData()
        if m.hasFormat("application/x-qabstractitemmodeldatalist"):
            data = m.data("application/x-qabstractitemmodeldatalist")
            r = self.decodeMimeData(data)
            AxesListWidget.last_drop = []
            for k in r.keys():
                AxesListWidget.last_drop.append(k)
        super(AxesListWidget, self).dropEvent(event)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.


from qtpy import QtGui, QtCore, QtWidgets


class ColorButton(QtWidgets.QPushButton):

    colorChanged = QtCore.Signal([QtGui.QColor])

    def __init__(self, color=None, parent=None):
        super(ColorButton, self).__init__(parent)
        self.setMinimumWidth(50)
        if color is None:
            color = QtGui.QColor('gray')
        self.color = color
        self.clicked.connect(self.choose_color)

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, color):
        self._color = color
        self.colorChanged.emit(color)
        self.update()

    def choose_color(self):
        color = QtWidgets.QColorDialog.getColor(self.color, self)
        if color.isValid():
            self.color = color

    def paintEvent(self, event):
        super(ColorButton, self).paintEvent(event)
        padding = 5

        rect = event.rect()
        painter = QtGui.QPainter(self)
        painter.setBrush(QtGui.QBrush(self.color))
        painter.setPen(QtGui.QColor("#CECECE"))
        rect.adjust(
            padding, padding,
            -1-padding, -1-padding)
        painter.drawRect(rect)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.

from hyperspyui.plugins.plugin import Plugin

from functools import partial
from collections import OrderedDict

from qtpy import QtGui, QtCore, QtWidgets
from qtpy.QtWidgets import (QApplication, QPushButton, QLabel, QGroupBox,
                            QHBoxLayout, QVBoxLayout, QComboBox)

from qtpy.QtGui import QPalette


from pyqode.core import api
from pyqode.core import modes
from pyqode.core import panels
from pyqode.core.backend import server

from hyperspyui.widgets.extendedqwidgets import ExToolWindow
from hyperspyui.widgets.colorpicker import ColorButton
from hyperspyui.util import block_signals


def tr(text):
    return QtCore.QCoreApplication.translate("Style", text)


class StylePlugin(Plugin):
    name = "Style"

    def __init__(self, *args, **kwargs):
        super(StylePlugin, self).__init__(*args, **kwargs)
        self.editor = None
        self.settings.set_default('_style', '')
        self.apply_styles()

    def create_actions(self):
        self.add_action('edit_style', tr("Edit styles"),
                        self.edit_dialog,
                        # icon='.svg',
                        tip=tr("Edit the application styles"))

    def create_menu(self):
        self.add_menuitem('Settings', self.ui.actions['edit_style'])

    def apply_styles(self):
        palette = self.settings['_palette']
        if palette is not None:
            QApplication.setPalette(QPalette(palette))
        QApplication.instance().setStyleSheet(self.settings['_style'] or '')

    def edit_dialog(self):
        if self.editor is not None:
            self.editor.show()
        else:
            self.editor = StyleDialog(self, self.ui, self.ui)
            self.editor.show()


class StyleDialog(ExToolWindow):
    """A dialog for editing Qt palette and stylesheet.

    Edits and sets Qt's application wide palette and stylesheet.
    See http://doc.qt.io/qt-4.8/stylesheet.html,
    and http://doc.qt.io/qt-4.8/qpalette.html
    for documentation of the effects of various settings, and
    the style sheet syntax.
    """

    palette_entries = OrderedDict((
        ('basic', (
            'Window',
            'WindowText',
        )),
        ('extended', (
            'Base',
            'Text',
            'ToolTipBase',
            'ToolTipText',
            'Button',
            'ButtonText',
        )),
        ('full', (
            'Highlight',
            'HighlightedText',
            'BrightText',
            'Light',
            'Midlight',
            'Mid',
            'Dark',
            'Shadow',
            'Link',
            'LinkVisited',
            'NoRole',
        )),
    ))

    pairs = [
        ('Window', 'WindowText'),
        ('Base', 'Text'),
        ('Button', 'ButtonText'),
        ('ToolTipBase', 'ToolTipText'),
        ('Highlight', 'HighlightedText'),
        ('Light', 'Midlight'),
        ('Mid', 'Dark'),
        ('Link', 'LinkVisited'),
    ]

    def __init__(self, plugin, ui, parent):
        super(StyleDialog, self).__init__(parent)
        self.ui = ui
        self.setWindowTitle(tr("Edit application styles"))
        self.plugin = plugin
        self._palette = QPalette()
        self._rows = {}
        i = 0
        for entries in self.palette_entries.values():
            for key in entries:
                self._rows[key] = i
                for pair in self.pairs:
                    if key == pair[0]:
                        break
                else:
                    i += 1
        self.create_controls()

        self.save_action = QtWidgets.QAction(self)
        self.save_action.setShortcut(QtGui.QKeySequence.Save)
        self.save_action.triggered.connect(self.save)
        self.addAction(self.save_action)

        self.load()

    def save(self):
        """Store the palette and stylesheet to the plugin's settings"""
        self.plugin.settings['_style'] = self.editor.toPlainText()
        self.plugin.settings['_palette'] = self._palette

    def apply(self):
        """Apply current palette and stylesheet application wide"""
        QApplication.setPalette(self._palette)
        QApplication.instance().setStyleSheet(self.editor.toPlainText())

    def load(self):
        """Load palette and stylesheet from the plugin't settings"""
        self.editor.setPlainText(
            self.plugin.settings['_style'] or '',
            'text/plain', 'utf8'
        )
        palette = self.plugin.settings['_palette']
        if palette is not None:
            self._palette = QPalette(palette)
        self.update_palette_controls()

    def _clear_styles(self):
        """Reset style inputs to default values"""
        self._palette = QPalette(self.style().standardPalette())
        self.editor.clear()
        self.update_palette_controls()

    def create_controls(self):
        """Build the editor's controls"""
        # First set up the stylesheet editor:
        editor = api.CodeEdit()
        editor.backend.start(server.__file__)

        editor.panels.append(panels.LineNumberPanel())
        editor.panels.append(panels.CheckerPanel())

        editor.modes.append(AutoIndentMode())
        editor.modes.append(modes.CaretLineHighlighterMode())
        editor.modes.append(modes.CodeCompletionMode())
        editor.modes.append(modes.ExtendedSelectionMode())
        editor.modes.append(modes.SmartBackSpaceMode())
        editor.modes.append(modes.OccurrencesHighlighterMode())
        editor.modes.append(modes.SymbolMatcherMode())
        editor.modes.append(modes.ZoomMode())

        self.editor = editor

        # Create the pushbuttons for the button bar:
        self.btn_apply = QPushButton(tr("Apply"))
        self.btn_apply.clicked.connect(self.apply)

        self.btn_save = QPushButton(tr("Save"))
        self.btn_save.clicked.connect(self.save)

        self.btn_revert = QPushButton(tr("Revert"))
        self.btn_revert.clicked.connect(self.load)

        self.btn_clear = QPushButton(tr("Clear"))
        self.btn_clear.clicked.connect(self._clear_styles)

        self.hbox = QHBoxLayout()
        for w in [self.btn_apply, self.btn_save,
                  self.btn_revert, self.btn_clear]:
            self.hbox.addWidget(w)

        # Create the group-box with the palette editor
        self.palette_box = self.create_palette_colors()

        # Lay out the various components:
        vbox = QVBoxLayout(self)
        vbox.addWidget(self.palette_box)
        vbox.addWidget(editor)
        vbox.addLayout(self.hbox)

        self.setLayout(vbox)

    def create_palette_colors(self):
        """Create the controls for editing the palette"""
        layout = QtWidgets.QGridLayout()

        # Create combobox simple/extended/full
        cbo = QComboBox()
        cbo.addItems(list(self.palette_entries.keys()))
        cbo.currentIndexChanged[str].connect(self._on_cbo_change)
        layout.addWidget(cbo, 0, 0)
        self.cbo_mode = cbo

        # Create pickers for all
        self.pickers = {}
        for key, entries in self.palette_entries.items():
            self.pickers[key] = {}
            for subkey in entries:
                btn = ColorButton()
                self.pickers[key][subkey] = btn
                btn.colorChanged.connect(
                    partial(self._on_color_pick, subkey))
                label = QLabel(subkey)
                if key != 'basic':
                    btn.hide()
                    label.hide()
                row = 1 + self._rows[subkey]
                for pair in self.pairs:
                    if subkey == pair[1]:
                        layout.addWidget(label, row, 2)
                        layout.addWidget(btn, row, 3)
                        break
                else:
                    layout.addWidget(label, row, 0)
                    layout.addWidget(btn, row, 1)

        # Initialize colors:
        self.update_palette_controls()

        box = QGroupBox('Palette')
        box.setLayout(layout)
        return box

    def update_palette_controls(self):
        """Update palette controls from internal QPalette"""
        for key, entries in self.palette_entries.items():
            for subkey in entries:
                color = self._palette.color(
                    getattr(self._palette, subkey))
                btn = self.pickers[key][subkey]
                with block_signals(btn):
                    btn.color = color

    def _setvis_palette_entry(self, key, show):
        """Show/hide specific palette entries"""
        row = 1 + self._rows[key]
        layout = self.palette_box.layout()
        # Always show/hide pairs together:
        for i in range(4):
            item = layout.itemAtPosition(row, i)
            if item is not None:
                item.widget().setVisible(show)

    def _on_color_pick(self, key, color):
        """Callback when a palette color has been picked"""
        if self.cbo_mode.currentText() == 'basic':
            # Other values should be auto-generated
            self._palette = QPalette(self.pickers['basic']['Window'].color)

        self._palette.setColor(
            getattr(self._palette, key),
            color
        )

    def _on_cbo_change(self, selection):
        """Callback for when palette-mode selection changes"""
        # Use all up-until selection (keys are ordered)
        included = list(self.palette_entries.keys())
        included = included[:1 + included.index(selection)]

        for key, entries in self.palette_entries.items():
            visible = key in included
            for subkey in entries:
                self._setvis_palette_entry(subkey, show=visible)


class AutoIndentMode(modes.AutoIndentMode):
    """
    Provides automatic stylesheet-specific auto indentation.
    """
    def _get_indent(self, cursor):
        text = cursor.block().text().strip()
        pre, post = super(AutoIndentMode, self)._get_indent(cursor)
        print(repr(text), repr(pre), repr(post))
        if text.endswith('{'):
            post += self.editor.tab_length * ' '
        elif text.startswith('}'):
            pre = pre[self.editor.tab_length:]
        return pre, post



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Fri Dec 12 23:43:54 2014

@author: Vidar Tonaas Fauske
"""

from qtpy import QtCore
from qtpy.QtWidgets import QMessageBox

import hyperspy.api as hs

from hyperspyui.plugins.plugin import Plugin
from hyperspyui.threaded import ProgressThreaded


def tr(text):
    return QtCore.QCoreApplication.translate("FFT", text)


class FFT_Plugin(Plugin):
    name = 'FFT'

    def create_actions(self):
        self.add_action('fft', "FFT", self.fft,
                        icon='fft.svg',
                        tip="Perform a fast fourier transform on the " +
                        "active part of the signal",
                        selection_callback=self.ui.select_signal)

        self.add_action('live_fft', "Live FFT", self.live_fft,
                        icon='live_fft.svg',
                        tip="Perform a fast fourier transform on the " +
                        "active part of the signal, and link it to the " +
                        "navigator",
                        selection_callback=self.ui.select_signal)

        self.add_action('ifft', "Inverse FFT", self.ifft,
                        icon='ifft.svg',
                        tip="Perform an inverse fast fourier transform on " +
                            "the active part of the signal",
                        selection_callback=self.ui.select_signal)

        self.add_action('get_real_image', "Real and imaginary",
                        self.get_real_image,
                        icon=None,
                        tip="Decompose a fast fourier transformation result " +
                        "into its real and imaginary part.",
                        selection_callback=self.ui.select_signal)

        self.add_action('get_amplitude_phase', "Amplitude and phase",
                        self.get_amplitude_phase,
                        icon=None,
                        tip="Decompose a fast fourier transformation result " +
                        "into its amplitude and phase.",
                        selection_callback=self.ui.select_signal)

    def create_menu(self):
        self.add_menuitem("Math", self.ui.actions['fft'])
        self.add_menuitem("Math", self.ui.actions['live_fft'])
        self.add_menuitem("Math", self.ui.actions['ifft'])
        self.add_menuitem("Math", self.ui.actions['get_real_image'])
        self.add_menuitem("Math", self.ui.actions['get_amplitude_phase'])

    def create_toolbars(self):
        self.add_toolbar_button("Math", self.ui.actions['fft'])
        self.add_toolbar_button("Math", self.ui.actions['live_fft'])
        self.add_toolbar_button("Math", self.ui.actions['ifft'])

    def fft(self, signals=None, shift=True, power_spectrum=True, inverse=False,
            on_complete=None):
        if signals is None:
            signals = self.ui.get_selected_signals()
        if isinstance(signals, hs.signals.BaseSignal):
            signals = (signals,)

        fftsignals = []

        def do_ffts():
            for i, signal in enumerate(signals):
                if inverse:
                    fft = signal.ifft(shift=shift)
                else:
                    fft = signal.fft(shift=shift)
                fftsignals.append(fft)
                yield i + 1

        def on_ffts_complete():
            for fs in fftsignals:
                fs.plot(power_spectrum=power_spectrum)
                sw = self.ui.lut_signalwrapper[fs]
                if on_complete is not None:
                    on_complete(sw)

        if len(signals) > 1:
            if inverse:
                label = tr('Performing inverse FFT')
            else:
                label = tr('Performing FFT')
            t = ProgressThreaded(self.ui,
                                 do_ffts(),
                                 on_ffts_complete,
                                 label=label,)
                                 # This breaks the progress bar...
                                 # generator_N=len(signals))
            t.run()
        else:
            for i in do_ffts():
                pass
            on_ffts_complete()

    def live_fft(self, signals=None, shift=True, power_spectrum=True):
        """
        The live FFT dynamically calculates the FFT as the user navigates.
        """
        if signals is None:
            signals = self.ui.get_selected_signals()
            if signals is None:
                return
        # Make sure we can iterate
        if isinstance(signals, hs.signals.BaseSignal):
            signals = (signals,)

        if len(signals) < 1:
            return

        s = signals[0]

        if isinstance(s, hs.signals.Signal2D):
            extent = s.axes_manager.signal_extent
            left = (extent[1] - extent[0]) / 4 + extent[0]
            right = 3 * (extent[1] - extent[0]) / 4 + extent[0]
            top = (extent[3] - extent[2]) / 4 + extent[2]
            bottom = 3 * (extent[3] - extent[2]) / 4 + extent[2]
            roi = hs.roi.RectangularROI(left, top, right, bottom)
        elif isinstance(s, hs.signals.Signal1D):
            extent = s.axes_manager.signal_extent
            half_range = (extent[1] - extent[0]) / 4
            roi = hs.roi.SpanROI(half_range + extent[0],
                                 3 * half_range + extent[0])
        else:
            mb = QMessageBox(QMessageBox.Information,
                             tr("Live FFT"),
                             tr("Only Signal2D and Signal1D are supported."),
                             QMessageBox.Ok)
            mb.exec_()

        roi.add_widget(s)
        roi_signal = roi.interactive(s, recompute_out_event=False)
        s_roi_fft = hs.interactive(roi_signal.fft,
                                   event=roi.events.changed,
                                   recompute_out_event=False,
                                   shift=shift)

        s_roi_fft.plot(power_spectrum=power_spectrum)

    def ifft(self, signals=None):
        return self.fft(signals, inverse=True)

    def get_real_image(self, signal=None):
        if signal is None:
            signal = self.ui.get_selected_signal()
        if not isinstance(signal,
                          hs.signals.ComplexSignal):
            mb = QMessageBox(QMessageBox.Information,
                             tr("Real and imaginary"),
                             tr("A complex signal is required."),
                             QMessageBox.Ok)
            mb.exec_()
        signal.real.plot()
        signal.imag.plot()

    def get_amplitude_phase(self, signal=None):
        if signal is None:
            signal = self.ui.get_selected_signal()
        if not isinstance(signal,
                          hs.signals.ComplexSignal):
            mb = QMessageBox(QMessageBox.Information,
                             tr("Amplitude and phase"),
                             tr("A complex signal is required."),
                             QMessageBox.Ok)
            mb.exec_()
        signal.amplitude.plot()
        signal.phase.plot()




import pytest


@pytest.mark.timeout(60)
def test_main_window(mainwindow):
    mainwindow.show()
    mainwindow.load_complete.emit()

    assert mainwindow.isVisible()
    assert mainwindow.windowTitle() == 'HyperSpy'



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Sat Feb 21 12:03:33 2015

@author: Vidar Tonaas Fauske
"""

from qtpy import QtCore

from .plugincreator import create_plugin_code


class Recorder(QtCore.QObject):
    record = QtCore.Signal(str)

    def __init__(self):
        super(Recorder, self).__init__()

        self.steps = list()
        self.pause_recording = False
        self.filter = {'actions': True, 'code': True}

    # ------ Recording API ------
    def add_code(self, code):
        if self.filter['code'] and not self.pause_recording:
            step = ('code', code.rstrip('\n'))
            self.steps.append(step)
            self._on_record(step)

    def add_action(self, action_key):
        if self.filter['actions'] and not self.pause_recording:
            step = ('action', action_key)
            self.steps.append(step)
            self._on_record(step)

    # ------ Event processing ------
    def _on_record(self, step):
        self.record.emit(self.step_to_code(step))

    # ------ Output API ------
    @staticmethod
    def step_to_code(step):
        if step[0] == 'code':
            return step[1] + '\n'
        elif step[0] == 'action':
            return "ui.actions['{0}'].trigger()\n".format(step[1])

    def to_code(self):
        code = ""
        for step in self.steps:
            code += self.step_to_code(step)
        return code

    def to_plugin(self, name, category=None, menu=False, toolbar=False,
                  icon=None):
        code = "ui = self.ui\n"
        code += "siglist = ui.hspy_signals\n"
        code += self.to_code()

        return create_plugin_code(code, name, category, menu, toolbar, icon)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Sun Mar 01 03:24:48 2015

@author: Vidar Tonaas Fauske
"""

import sys
import os

from qtpy import QtCore
from qtpy.QtCore import Qt, QModelIndex
from qtpy.QtWidgets import (QDialogButtonBox, QTableView, QHeaderView,
                            QPushButton, QVBoxLayout)

from .extendedqwidgets import ExToolWindow
from .editorwidget import EditorWidget


def tr(text):
    return QtCore.QCoreApplication.translate("PluginManagerWidget", text)


class PluginsModel(QtCore.QAbstractItemModel):

    EnabledColumn = 0
    NameColumn = 1
    PathColumn = 2

    def __init__(self, plugin_manager, parent=None):
        super(PluginsModel, self).__init__(parent)
        self.plugin_manager = plugin_manager
        self._update_data()

    def _update_data(self):
        self._plugin_data = []
        for i, (name, (enabled, ptype)) in enumerate(
                self.plugin_manager._enabled.items()):
            path = sys.modules[ptype.__module__].__file__
            path = os.path.normpath(path)
            if path.endswith('.pyc') or path.endswith('.pyo'):
                path = path[:-1]
            self._plugin_data.append([enabled, name, path])

    def flags(self, index):
        if index.column() in [self.NameColumn, self.PathColumn]:
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        elif index.column() == self.EnabledColumn:
            return (Qt.ItemIsEditable | Qt.ItemIsUserCheckable |
                    Qt.ItemIsEnabled)

    def parent(self, index):
        return QModelIndex()

    def index(self, row, column, parent=QModelIndex()):
        return self.createIndex(row, column)

    def data(self, index, role=Qt.DisplayRole):
        self._update_data()
        r, c = index.row(), index.column()
        if role in (Qt.DisplayRole, Qt.EditRole):
            if c == self.EnabledColumn:
                return None
            else:
                return self._plugin_data[r][c]
        elif role == Qt.CheckStateRole:
            if c == self.EnabledColumn:
                if self._plugin_data[r][c]:
                    return Qt.Checked
                else:
                    return Qt.Unchecked
        return None

    def setData(self, index, value, role=Qt.EditRole):
        r, c = index.row(), index.column()
        if role in (Qt.DisplayRole, Qt.EditRole):
            if c == self.EnabledColumn:
                name = self._plugin_data[r][self.NameColumn]
                self._plugin_data[r][c] = bool(value)
                self.plugin_manager.enable_plugin(name, bool(value))
                return True
        if role == Qt.CheckStateRole:
            if c == self.EnabledColumn:
                name = self._plugin_data[r][self.NameColumn]
                enabled = (value != Qt.Unchecked)
                self._plugin_data[r][c] = enabled
                self.plugin_manager.enable_plugin(name, enabled)
                return True
        return False

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                if section == self.EnabledColumn:
                    return tr("Enabled")
                elif section == self.NameColumn:
                    return tr("Name")
                elif section == self.PathColumn:
                    return tr("Path")
        return None

    def rowCount(self, parent=QtCore.QModelIndex()):
        return len(self._plugin_data)

    def columnCount(self, parent=QtCore.QModelIndex()):
        return 3


class PluginManagerWidget(ExToolWindow):

    def __init__(self, plugin_manager, parent=None):
        super(PluginManagerWidget, self).__init__(parent)

        self.setWindowTitle(tr("Plugin manager"))
        self.plugin_manager = plugin_manager
        self.create_controls()

    def edit_plugin(self):
        i = self.table.currentIndex()
        if not i.isValid():
            return
        if i.column() != PluginsModel.PathColumn:
            i = i.sibling(i.row(), PluginsModel.PathColumn)
        path = self.model.data(i)
        if path is not None and os.path.isfile(path):
            ui = self.plugin_manager.ui
            e = EditorWidget(ui, ui, path)
            ui.editors.append(e)
            e.is_plugin = True
            e.show()

    def create_controls(self):
        table = QTableView(self)
        self.model = PluginsModel(self.plugin_manager)
        table.setModel(self.model)
        h = table.horizontalHeader()
        h.setSectionResizeMode(QHeaderView.ResizeToContents)
        table.setHorizontalHeader(h)
        h = table.verticalHeader()
        h.setSectionResizeMode(QHeaderView.ResizeToContents)
        table.setVerticalHeader(h)
        self.table = table
        width = 80
        for i in range(3):
            width += table.columnWidth(i)

        btns = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
                                Qt.Horizontal)
        self.edit_btn = QPushButton("Edit")
        btns.addButton(self.edit_btn, QDialogButtonBox.ActionRole)
        self.edit_btn.clicked.connect(self.edit_plugin)
        btns.accepted.connect(self.accept)
        btns.rejected.connect(self.reject)

        vbox = QVBoxLayout()
        vbox.addWidget(table)
        vbox.addWidget(btns)
        self.setLayout(vbox)
        s = self.size()
        s.setHeight(table.rowHeight(0) * 10)
        s.setWidth(width)
        self.resize(s)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Sun Mar 01 15:20:55 2015

@author: Vidar Tonaas Fauske
"""

from hyperspyui.plugins.plugin import Plugin

from qtpy import QtCore, QtWidgets

from hyperspyui.widgets.elementpicker import ElementPickerWidget
from hyperspyui.util import SignalTypeFilter
from hyperspyui.tools import SignalFigureTool

import hyperspy.signals
from hyperspy.misc.eds.utils import _get_element_and_line
import numpy as np

import os
from functools import partial


def tr(text):
    return QtCore.QCoreApplication.translate("BasicSpectrumPlugin", text)


class Namespace:
    pass


QWIDGETSIZE_MAX = 16777215


class BasicSpectrumPlugin(Plugin):
    name = "Basic spectrum tools"

    def create_actions(self):
        self.add_action(
            'plot_components', tr("Plot components"),
            self.plot_components,
            icon=None,
            tip=tr("Plot the individual components of the model"),
            selection_callback=self._plot_components_state_update)
        self.actions['plot_components'].setCheckable(True)

        self.add_action(
            'adjust_component_position', tr("Adjust component positions"),
            self.adjust_component_position,
            icon=None,
            tip=tr("Adjust the position of the components interactively"),
            selection_callback=self._adjust_components_state_update)
        self.actions['adjust_component_position'].setCheckable(True)

        self.add_action(
            'remove_background', tr("Remove Background"),
            self.remove_background,
            icon='power_law.svg',
            tip=tr("Interactively define the background, and remove it"),
            selection_callback=SignalTypeFilter(
                hyperspy.signals.Signal1D, self.ui))

        self.add_action('fourier_ratio', tr("Fourier Ratio Deconvoloution"),
                        self.fourier_ratio,
                        icon='fourier_ratio.svg',
                        tip=tr("Use the Fourier-Ratio method to deconvolve "
                               "one signal from another"),
                        selection_callback=SignalTypeFilter(
                            hyperspy.signals.EELSSpectrum, self.ui))

        self.add_action('estimate_thickness', tr("Estimate thickness"),
                        self.estimate_thickness,
                        icon="t_over_lambda.svg",
                        tip=tr("Estimates the thickness (relative to the "
                               "mean free path) of a sample using the "
                               "log-ratio method."),
                        selection_callback=SignalTypeFilter(
                            hyperspy.signals.EELSSpectrum, self.ui))

        # -------------- Filter actions -----------------

        self.add_action('smooth_savitzky_golay', tr("Smooth Savitzky-Golay"),
                        self.smooth_savitzky_golay,
                        icon=None,
                        tip=tr("Apply a Savitzky-Golay filter"),
                        selection_callback=SignalTypeFilter(
                            hyperspy.signals.Signal1D, self.ui))

        self.add_action('smooth_lowess', tr("Smooth Lowess"),
                        self.smooth_lowess,
                        icon=None,
                        tip=tr("Apply a Lowess smoothing filter"),
                        selection_callback=SignalTypeFilter(
                            hyperspy.signals.Signal1D, self.ui))

        self.add_action('smooth_tv', tr("Smooth Total variation"),
                        self.smooth_tv,
                        icon=None,
                        tip=tr("Total variation data smoothing"),
                        selection_callback=SignalTypeFilter(
                            hyperspy.signals.Signal1D, self.ui))

        self.add_action('filter_butterworth', tr("Butterworth filter"),
                        self.filter_butterworth,
                        icon=None,
                        tip=tr("Apply a Butterworth filter"),
                        selection_callback=SignalTypeFilter(
                            hyperspy.signals.Signal1D, self.ui))

        self.add_action('hanning_taper', tr("Hanning taper"),
                        self.hanning_taper,
                        icon=None,
                        tip=tr("Apply a Hanning taper to both ends of the "
                               "data."),
                        selection_callback=SignalTypeFilter(
                            hyperspy.signals.Signal1D, self.ui))

    def create_menu(self):
        self.add_menuitem("Model", self.ui.actions['plot_components'])
        self.add_menuitem("Model",
                          self.ui.actions['adjust_component_position'])
        self.add_menuitem("EELS", self.ui.actions['remove_background'])
        self.add_menuitem('EELS', self.ui.actions['fourier_ratio'])
        self.add_menuitem('EELS', self.ui.actions['estimate_thickness'])
        self.add_menuitem("Spectrum", self.ui.actions['smooth_savitzky_golay'])
        self.add_menuitem("Spectrum", self.ui.actions['smooth_lowess'])
        self.add_menuitem("Spectrum", self.ui.actions['smooth_tv'])
        self.add_menuitem("Spectrum", self.ui.actions['filter_butterworth'])
        self.add_menuitem("Spectrum", self.ui.actions['hanning_taper'])

    def create_toolbars(self):
        self.add_toolbar_button("EELS", self.ui.actions['remove_background'])
        self.add_toolbar_button("EELS", self.ui.actions['fourier_ratio'])
        self.add_toolbar_button("EELS", self.ui.actions['estimate_thickness'])

    def create_tools(self):
        try:
            # Import for functionality test
            from hyperspy.misc.eds.utils import get_xray_lines_near_energy as _
            self.picker_tool = ElementPickerTool()
            self.picker_tool.picked[str].connect(self.pick_element)
            self.add_tool(self.picker_tool,
                          SignalTypeFilter(
                              (  # hyperspy.signals.EELSSpectrum,
                               hyperspy.signals.EDSSEMSpectrum,
                               hyperspy.signals.EDSTEMSpectrum),
                              self.ui))
        except ImportError:
            pass

    def _toggle_fixed_height(self, floating):
        w = self.picker_widget
        if floating:
            w.setFixedHeight(QWIDGETSIZE_MAX)
        else:
            w.setFixedHeight(w.minimumSize().height())

    def create_widgets(self):
        self.picker_widget = ElementPickerWidget(self.ui, self.ui)
        self.picker_widget.topLevelChanged[bool].connect(
            self._toggle_fixed_height)
        self.add_widget(self.picker_widget)
        self._toggle_fixed_height(False)

    def pick_element(self, element, signal=None):
        wp = self.picker_widget
        if signal:
            wp.set_signal(signal)
        wp.set_element(element, True)
        if not wp.chk_markers.isChecked():
            wp.chk_markers.setChecked(True)

    def _plot_components_state_update(self, win, action):
        model = self.ui.get_selected_model()
        action.setEnabled(model is not None)
        if model is not None:
            action.setChecked(model._plot_components)

    def plot_components(self, model=None):
        """
        Plot the function of each component together with the model.
        """
        model = model or self.ui.get_selected_model()
        if model is None:
            return
        self.record_code("model = ui.get_selected_model()")
        current = model._plot_components
        if current:
            model.disable_plot_components()
            self.record_code("model.disable_plot_components()")
        else:
            model.enable_plot_components()
            self.record_code("model.enable_plot_components()")

    def _adjust_components_state_update(self, win, action):
        model = self.ui.get_selected_model()
        action.setEnabled(model is not None)
        if model is not None:
            action.setChecked(bool(model._position_widgets))

    def adjust_component_position(self, model=None):
        """
        Add widgets to adjust the position of the components in the model.
        """
        model = model or self.ui.get_selected_model()
        if model is None:
            return
        self.record_code("model = ui.get_selected_model()")
        current = bool(model._position_widgets)
        if current:
            model.disable_adjust_position()
            self.record_code("model.disable_adjust_position()")
        else:
            model.enable_adjust_position()
            self.record_code("model.enable_adjust_position()")

    def fourier_ratio(self):
        signals = self.ui.select_x_signals(2, [tr("Core loss"),
                                               tr("Low loss")])
        self.record_code("<p>.fourier_ratio()")
        if signals is None:
            return
        s_core, s_lowloss = signals
        title = s_core.name + "[Fourier-ratio]"
        s_core, s_lowloss = s_core.signal, s_lowloss.signal

        threshold = s_lowloss.estimate_elastic_scattering_threshold().data
        threshold = np.ma.masked_array(threshold, np.isnan(threshold)).mean()

        sdcnv = s_core.fourier_ratio_deconvolution(s_lowloss,
                                                   threshold=threshold)
        sdcnv.data = np.ma.masked_array(
            sdcnv.data,
            mask=(np.isnan(sdcnv.data) | np.isinf(sdcnv.data)))
        sdcnv.metadata.General.title = title
        sdcnv.plot()

    def remove_background(self, signal=None):
        signal = signal or self.ui.get_selected_signal()
        signal.remove_background()

    def estimate_thickness(self, signal=None):
        signal = signal or self.ui.get_selected_signal()
        threshold = signal.estimate_elastic_scattering_threshold().data
        threshold = np.ma.masked_array(threshold, np.isnan(threshold)).mean()
        s_t = signal.estimate_thickness(threshold)
        s_t.plot()
        self.record_code("signal = ui.get_selected_signal()")
        self.record_code(
            "threshold = signal.estimate_elastic_scattering_threshold().data")
        self.record_code(
            "threshold = np.ma.masked_array(threshold, "
            "np.isnan(threshold)).mean()")
        self.record_code("thickness = signal.estimate_thickness(threshold)")

    # ----------- Filter callbacks --------------

    def smooth_savitzky_golay(self, signal=None):
        signal = signal or self.ui.get_selected_signal()
        if signal is not None:
            signal.smooth_savitzky_golay()
            self.record_code("signal = ui.get_selected_signal()")
            self.record_code("signal.smooth_savitzky_golay()")

    def smooth_lowess(self, signal=None):
        signal = signal or self.ui.get_selected_signal()
        if signal is not None:
            signal.smooth_lowess()
            self.record_code("signal = ui.get_selected_signal()")
            self.record_code("signal.smooth_lowess()")

    def smooth_tv(self, signal=None):
        signal = signal or self.ui.get_selected_signal()
        if signal is not None:
            signal.smooth_tv()
            self.record_code("signal = ui.get_selected_signal()")
            self.record_code("signal.smooth_tv()")

    def filter_butterworth(self, signal=None):
        signal = signal or self.ui.get_selected_signal()
        if signal is not None:
            signal.filter_butterworth()
            self.record_code("signal = ui.get_selected_signal()")
            self.record_code("signal.filter_butterworth()")

    def hanning_taper(self, signal=None):
        signal = signal or self.ui.get_selected_signal()
        if signal is not None:
            signal.hanning_taper()
            self.record_code("signal = ui.get_selected_signal()")
            self.record_code("signal.hanning_taper()")


class ElementPickerTool(SignalFigureTool):
    picked = QtCore.Signal(str)

    def __init__(self, windows=None):
        super(ElementPickerTool, self).__init__(windows)
        self.ranged = False
        self.valid_dimensions = [1]

    def get_name(self):
        return "Element picker tool"

    def get_category(self):
        return 'EDS'

    def get_icon(self):
        return os.path.dirname(__file__) + '/../images/periodic_table.svg'

    def on_pick_line(self, line):
        el, _ = _get_element_and_line(line)
        self.picked.emit(el)

    def is_selectable(self):
        return True

    def on_mouseup(self, event):
        if event.inaxes is None:
            return
        energy = event.xdata
        axes = self._get_axes(event)
        if len(axes) not in self.valid_dimensions:
            return
        a = axes[0]
        if a.units.lower() == 'ev':
            energy /= 1000.0
        from hyperspy.misc.eds.utils import get_xray_lines_near_energy
        lines = get_xray_lines_near_energy(energy)
        if lines:
            m = QtWidgets.QMenu()
            for line in lines:
                m.addAction(line, partial(self.on_pick_line, line))
            m.exec_(QCursor.pos())



from hyperspyui.plugins.plugin import Plugin
import numpy as np

from hyperspyui.util import win2sig


class EnsureLt2D(object):

    def __init__(self, ui):
        self.ui = ui

    def __call__(self, win, action):
        sig = win2sig(win, self.ui.signals, self.ui._plotting_signal)
        valid = sig is not None and (
            len(sig.signal.data.shape) <= 2 or (
                win is sig.signal_plot and
                sig.signal.axes_manager.signal_dimension <= 2))

        action.setEnabled(valid)


class SaveDataAsText(Plugin):
    name = "Save data as text"

    def create_actions(self):
        self.add_action(self.name + '.save_as_text', self.name,
                        self.save_as_text,
                        tip="",
                        selection_callback=EnsureLt2D(self.ui))

    def create_menu(self):
        self.add_menuitem('File', self.ui.actions[self.name + '.save_as_text'])

    def save_as_text(self, signal=None):
        if signal is None:
            signal = self.ui.get_selected_signal()
        # What to save
        if len(signal.data.shape) <= 2:
            data = signal.data
        elif signal.axes_manager.signal_dimension <= 2:
            data = signal().data
        else:
            raise ValueError("Ivalid signal dimensions for saving as text.")

        extensions = "Text file (*.txt);;*.csv;;*.dat;;All files (*.*)"
        filename = self.ui.prompt_files(extensions, exists=False)
        if filename:
            np.savetxt(filename, data, delimiter=',\t')



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Tue Nov 04 16:04:17 2014

@author: Vidar Tonaas Fauske
"""
import sys

try:
    from qtconsole.rich_jupyter_widget import RichJupyterWidget
    from qtconsole.inprocess import QtInProcessKernelManager
except ImportError:
    from IPython.qt.console.rich_ipython_widget import RichIPythonWidget as \
        RichJupyterWidget
    from IPython.qt.inprocess import QtInProcessKernelManager
from IPython.lib import guisupport


def _init_asyncio_patch():
    """set default asyncio policy to be compatible with tornado
    Tornado 6 (at least) is not compatible with the default
    asyncio implementation on Windows

    Pick the older SelectorEventLoopPolicy on Windows
    if the known-incompatible default policy is in use.
    do this as early as possible to make it a low priority and overrideable

    ref: https://github.com/tornadoweb/tornado/issues/2608
    FIXME: if/when tornado supports the defaults in asyncio,
           remove and bump tornado requirement for py38
    """
    if sys.platform.startswith("win") and sys.version_info >= (3, 8):
        import asyncio
        try:
            from asyncio import (
                WindowsProactorEventLoopPolicy,
                WindowsSelectorEventLoopPolicy,
            )
        except ImportError:
            pass
            # not affected
        else:
            if type(asyncio.get_event_loop_policy()) is WindowsProactorEventLoopPolicy:
                # WindowsProactorEventLoopPolicy is not compatible with tornado 6
                # fallback to the pre-3.8 default of Selector
                asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())


class ConsoleWidget(RichJupyterWidget):

    def __init__(self, *args, **kwargs):
        super(ConsoleWidget, self).__init__(*args, **kwargs)

        _init_asyncio_patch()
        # Create an in-process kernel
        app = guisupport.get_app_qt4()
        kernel_manager = QtInProcessKernelManager()
        kernel_manager.start_kernel()

        # Set the kernel data
        self.kernel = kernel_manager.kernel
        self.kernel.gui = 'qt'

        kernel_client = kernel_manager.client()
        kernel_client.start_channels()

        def stop():
            kernel_client.stop_channels()
            kernel_manager.shutdown_kernel()
            app.exit()

        self.kernel_manager = kernel_manager
        self.kernel_client = kernel_client
        self.exit_requested.connect(stop)

    def ex(self, source):
        self.kernel.shell.ex(source)

    def push(self, variables):
        self.kernel.shell.push(variables)

    def _execute(self, source, hidden):
        """ Execute 'source'. If 'hidden', do not show any output.

        See parent class :meth:`execute` docstring for full details.

        Overriden copy to move 'executing' event before it is actually executed
        """
        if not hidden:
            self.executing.emit(source)
        msg_id = self.kernel_client.execute(source, hidden)
        self._request_info['execute'][
            msg_id] = self._ExecutionRequest(msg_id, 'user')
        self._hidden = hidden



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Wed Jan 07 20:01:34 2015

@author: Vidar Tonaas Fauske
"""

from hyperspyui.widgets.extendedqwidgets import FigureWidget

from qtpy import QtWidgets


class TraitsWidget(FigureWidget):

    """
    DockWidget for TraitsUI dialogs. The default behavior is to update the
    dialog when the active subwindow changes or when the widget becomes visible
    (updates are supressed when widget is hidden). This is done by capturing
    the traitsui dialog as it is created (see code in mainwindowbase).
    """

    def __init__(
            self, main_window, cb_make_dialog, cb_check=None, parent=None):
        """
        Create a TraitsWidget. 'main_window' is the application main window,
        cb_make_dialog a callback that will raise the dialog (which will then
        be captured), cb_check is an optional callback which determines whether
        the given window is supported for this dialog.
        """
        self.cb_check = cb_check
        self.cb_make_dialog = cb_make_dialog
        super(TraitsWidget, self).__init__(main_window, parent)

    def clear_editor(self):
        """
        Empties out the dockwidget.
        """
        w = self.widget()
        if w is not None:
            w.close()
            self.setWidget(None)

    def _check(self, window):
        if window is None:
            return False
        elif self.cb_check is None:
            return True
        else:
            return self.cb_check(window)

    def _on_figure_change(self, window):
        """
        Called when a connected update triggers. If the window is valid, it
        sets up the traitsui dialog capture, and calls cb_make_dialog.  If the
        window is invalid, it clears the widget.
        """
        super(TraitsWidget, self)._on_figure_change(window)

        if self._check(window):
            self.ui.capture_traits_dialog(self.set_traits_editor)
            self.cb_make_dialog(window)
        else:
            self.clear_editor()

    def set_traits_editor(self, traits_dialog):
        """
        Called when a traitsui dialog has been captured.
        """
        self.clear_editor()
        # Fix resizing
        sp = traits_dialog.sizePolicy()
        sp.setVerticalPolicy(QtWidgets.QSizePolicy.Fixed)
        sp.setHorizontalPolicy(QtWidgets.QSizePolicy.Expanding)
        traits_dialog.layout().setSizeConstraint(
                QtWidgets.QLayout.SetDefaultConstraint)
        traits_dialog.setSizePolicy(sp)

        # Set widget
        self.setWidget(traits_dialog)
        traits_dialog.show()



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Sun Dec 07 03:48:36 2014

@author: Vidar Tonaas Fauske
"""

from qtpy import QtCore


class Tool(QtCore.QObject):

    def get_name(self):
        raise NotImplementedError()

    def get_category(self):
        return None

    def get_description(self):
        name = self.get_name()
        if name is None:
            return None
        if self.is_selectable():
            return "Select the " + name + "."
        elif self.single_action():
            return "Run the " + name + "."

    def get_icon(self):
        return None

    def single_action(self):
        return None

    def is_selectable(self):
        return False

    def connect(self, targets=None):
        pass

    def disconnect(self, targets=None):
        pass




import os
import tempfile
import shutil

from hyperspyui.version import __version__
from hyperspyui.__main__ import get_splash

import pytest

from qtpy import QtCore

# QtWebEngineWidgets must be imported before a QCoreApplication instance
# is created (used in eelsdb plugin)
# Avoid a bug in Qt: https://bugreports.qt.io/browse/QTBUG-46720
from qtpy import QtWebEngineWidgets

QCoreApplication = QtCore.QCoreApplication
QSettings = QtCore.QSettings

QCoreApplication.setApplicationName("HyperSpyUI-tests")
QCoreApplication.setOrganizationName("Hyperspy")
QCoreApplication.setApplicationVersion(__version__)

QSettings.setDefaultFormat(QSettings.IniFormat)


_tmpdirpath = ''


def pytest_configure(config):
    global _tmpdirpath
    _tmpdirpath = tempfile.mkdtemp()
    userpath = os.path.join(_tmpdirpath, 'user')
    syspath = os.path.join(_tmpdirpath, 'sys')
    os.mkdir(userpath)
    os.mkdir(syspath)

    QSettings.setPath(QSettings.IniFormat,
                      QSettings.UserScope, userpath)
    QSettings.setPath(QSettings.IniFormat,
                      QSettings.SystemScope, syspath)

    settings = QSettings()
    settings.setValue(
        'plugins/Version selector/check_for_updates_on_start', False)


def pytest_unconfigure(config):
    shutil.rmtree(_tmpdirpath)


@pytest.fixture(scope='session')
def mainwindow(qapp):
    from hyperspyui.mainwindow import MainWindow

    window = MainWindow(get_splash(), argv=[])
    yield window
    qapp.processEvents()
    window.close()
    window.deleteLater()
    del window
    qapp.processEvents()



from hyperspyui.plugins.plugin import Plugin
import numpy as np
import hyperspy.api as hs
from hyperspy.drawing.widgets import LabelWidget


class DmAnnotations(Plugin):
    name = "DM Annotations"

    def create_actions(self):
        self.add_action(self.name + '.default',
                        "Add DM Annotations",
                        self.add_annotations,
                        tip="Add annotations from DM metadata as markers")

    def create_menu(self):
        self.add_menuitem('Signal', self.ui.actions[self.name + '.default'])

    def add_annotations(self,
                        signal=None,
                        add_text=False,
                        plot_beam=True,
                        plot_si=True,
                        plot_drift=True):
        """
        Plot a hyperspy signal with the markers from digital micrograph enabled

        Parameters
        ----------
        signal : Signal
            DM signal
        add_text : bool
            Switch to control if labels for the markers are added to the plot
        plot_beam : bool
            Switch to control if beam point is plotted (if present)
        plot_si : bool
            Switch to control if spectrum image box or line is plotted (i
            present)
        plot_drift : bool
            Switch to control if spatial drift box is plotted (if present)
        text_size : str or float
            size of the text that will be written on the image (follows same
            convention as the `Text
            <http://matplotlib.org/1.3.0/api/artist_api.html#matplotlib.text.
            Text.set_size>`_ matplotlib Artist
        """

        if signal is None:
            signal = self.ui.get_selected_signal()

        annotation_list = (signal.original_metadata.DocumentObjectList.
                           TagGroup0.AnnotationGroupList)

        scale = signal.axes_manager[0].scale

        mapping = {
            'Beam': self._add_beam if plot_beam else self._dummy,
            'Beam (parked)': self._add_beam if plot_beam else self._dummy,
            'Spectrum Image': self._add_si if plot_si else self._dummy,
            'Spatial Drift': self._add_drift if plot_drift else self._dummy
        }

        for i in range(len(annotation_list)):
            try:
                label = annotation_list['TagGroup' + str(i)]['Label']
                loc = annotation_list['TagGroup' + str(i)]['Rectangle']
                scaled_loc = [scale * i for i in loc]
                mapping[label](signal, scaled_loc, add_text)

            except AttributeError:
                pass

    def _dummy(self, *args, **kwargs):
        pass

    def _add_label(self, image, text, color, x, y):
        print("adding label", text, color, x, y)
        label = LabelWidget(image.axes_manager)
        label.position = (x, y)
        label.text_color = color
        label.string = text
        label.set_mpl_ax(image._plot.signal_plot.ax)
        print(label.axes, label.position)

    def _add_beam(self, image, location, add_text):
        beam_m = hs.plot.markers.point(x=location[1],
                                       y=location[0],
                                       color='red')
        image.add_marker(beam_m)
        if add_text:
            self._add_label(image, 'Beam', 'red',
                            location[1] - 0.5, location[0] - 1.5)

    def _add_si(self, image, location, add_text):
        # adds a green rectangle (or line, if the coordinates are such) to
        # image
        si_m = hs.plot.markers.rectangle(x1=location[1],
                                         y1=location[0],
                                         x2=location[3],
                                         y2=location[2],
                                         color='#13FF00')
        image.add_marker(si_m)
        if add_text:
            self._add_label(image, 'Spectrum Image', '#13FF00',
                            location[1], location[0] - 0.5)

    def _add_drift(self, image, location, add_text):
        drift_m = hs.plot.markers.rectangle(x1=location[1],
                                            y1=location[0],
                                            x2=location[3],
                                            y2=location[2],
                                            color='yellow')
        image.add_marker(drift_m)
        if add_text:
            self._add_label(image, 'Spatial Drift', 'yellow',
                            location[1], location[0] - 0.5)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Wed Jan 07 19:56:18 2015

@author: Vidar Tonaas Fauske
"""

from hyperspyui.widgets.traitswidget import TraitsWidget
from hyperspyui.util import win2sig

from hyperspyui.plugins.plugin import Plugin


class AxesConf(Plugin):

    """
    Makes a widget that allows the configuration of the signal axes. The widget
    captures and displays the traitsui dialog shown by
    signal.axes_manager.show().
    """
    name = "Axes Configuration"

    def __init__(self, main_window):
        super(AxesConf, self).__init__(main_window)  # Sets self.ui
        self.widget = None

    def create_widgets(self):
        """
        Creates the TraitsWidget (inherits QDockWidget), and adds it to
        self.ui.
        """
        self.widget = TraitsWidget(self.ui, self.make_traits_dialog,
                                   self.valid_window, self.ui)
        self.widget.setWindowTitle("Axes configuration")
        self.widget.hide()  # Initial state hidden
        self.add_widget(self.widget)

    def valid_window(self, window):
        """
        Check whether the window belongs to a signalwrapper.
        """
        sig = win2sig(window)
        return sig is not None

    def make_traits_dialog(self, window):
        """
        Creates the taitsui dialog. The TraitsWidget captures and displays it.
        """
        sig = win2sig(window)
        sig.signal.axes_manager.gui()



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.

from hyperspyui.plugins.plugin import Plugin

from qtpy import QtCore, QtWidgets
from qtpy.QtWidgets import QDialogButtonBox
from qtpy.QtWidgets import (QCheckBox, QPushButton, QHBoxLayout, QVBoxLayout, 
                            QFormLayout, QDialog, QMessageBox, QLabel, QWidget,
                            QComboBox, QTextEdit)
from qtpy.QtCore import Qt

import os
import re
from collections import OrderedDict
import importlib
from functools import partial
from io import StringIO

from hyperspyui.util import block_signals


def check_git_repo(package_name):
    """
    Try to determine if the package "package_name" is installed as a source
    install from a git folder.
    """
    pkg_path = os.path.dname(importlib.util.find_spec(package_name).origin)
    while pkg_path:
        if os.path.exists(os.path.join(pkg_path, '.git')):
            return True
        pkg_path, dummy = os.path.split(pkg_path)
        if not dummy:
            break
    return False


def get_github_branches(repo_url):
    """
    Return a list of branches for the github repo at given URL.

    The branches are returned in the form of an ordered dictionary, with branch
    names as keys, and source zip URL as value.
    """
    import urllib.request
    branch_url = repo_url + '/branches/all'
    res = urllib.request.urlopen(branch_url)
    html = res.read().decode(res.headers.get_content_charset())
    names = re.findall(
        '<a href=".*?" class="branch-name css-truncate-target">(.*?)</a>',
        html)
    return OrderedDict(((n, repo_url + '/archive/%s.zip' % n) for n in names))

try:
    import git

    def check_git_cmd(prompt=True, parent=None):
        """
        Check if we have git.exe. If not prompt for its location (if `prompt`
        is True).
        """
        try:
            git.Git().status()
            return True
        except git.cmd.GitCommandNotFound:
            if prompt:
                ext_filt = '*.exe' if os.name == 'NT' else None
                path = QtWidgets.QFileDialog.getOpenFileName(
                    parent, tr('Specify git executable'),
                    git.Git.GIT_PYTHON_GIT_EXECUTABLE, ext_filt)
                # Pyside returns tuple, PyQt not
                if isinstance(path, (tuple, list)):
                    path = path[0]
                if path and os.path.exists(path):
                    git.Git.GIT_PYTHON_GIT_EXECUTABLE = path
                    return check_git_cmd(parent)
            return False

    def get_git_branches(package_name, fetch=False):
        """
        Get both local and remote branches for the local git repo of
        `package_name`. Ensure that the package has a local repo by calling
        `check_git_repo` first.
        """
        pkg_path = imp.find_module(package_name)[1]
        r = git.Repo(pkg_path, search_parent_directories=True)
        # Start with active branch:
        try:
            branches = OrderedDict(((r.active_branch.name, r.active_branch),))
        except TypeError:
            # active branch is detached
            branches = OrderedDict((("<Detached HEAD>", None),))
        # Add local branches:
        branches.update(((b.name, b) for b in r.heads))

        # Add remote branches:
        for remote in r.remotes:
            try:
                if fetch:
                    remote.fetch()
            except git.GitCommandError:
                continue
            # Don't include PRs in list
            branches.update(((b.name, b) for b in remote.refs if
                             '/pr/' not in b.name))
        return branches

    got_git = True
    use_git = True

except ImportError:
    got_git = False
    use_git = False


def get_branches(package_name, url):
    """
    Either get git branches, or fetch branches from github.com
    """
    if use_git and check_git_repo(package_name):
        return get_git_branches(package_name)
    else:
        return get_github_branches(url)


def checkout_branch(branch, stream=None):
    """
    If `branch` is a string, assume it is archive url. Try to install by pip.
    Otherwise `branch` is assumed by a branch object from the `git` package,
    which will be attempted to be checked out.
    """
    if branch is None:
        return
    if isinstance(branch, str):
        import pip
        import sys

        class WrapDownloadProgressBar(pip.download.DownloadProgressBar):
            def __init__(self, *args, **kwargs):
                super(WrapDownloadProgressBar, self).__init__(
                    *args, **kwargs)
                self.file = stream

        class WrapDownloadProgressBarSpinner(
                pip.download.DownloadProgressSpinner):
            def __init__(self, *args, **kwargs):
                super(WrapDownloadProgressBarSpinner, self).__init__(
                    *args, **kwargs)
                self.file = stream
        ic = pip.commands.InstallCommand()
        ic.log_streams = (stream, stream)
        old_classes = (pip.download.DownloadProgressBar,
                       pip.download.DownloadProgressSpinner)
        pip.download.DownloadProgressBar = WrapDownloadProgressBar
        pip.download.DownloadProgressSpinner = WrapDownloadProgressBarSpinner
        stdout_set = False
        if sys.__stdout__ is None:
            sys.__stdout__ = sys.stdout
            stdout_set = True
        try:
            ic.main(['--no-deps', '-I', branch])
            ic.main([branch])
        finally:
            if stdout_set:
                sys.__stdout__ = None  # Let's leave things as we found them.
            (pip.download.DownloadProgressBar,
             pip.download.DownloadProgressSpinner) = old_classes
    else:
        try:
            branch.checkout()
        except git.GitCommandError as e:
            mb = QMessageBox(QMessageBox.Critical, tr("Git checkout failed"),
                             e.stderr)
            mb.exec_()
            raise ValueError()


def tr(text):
    return QtCore.QCoreApplication.translate("GitSelector", text)


class GitSelector(Plugin):
    name = "Version selector"

    def __init__(self, main_window):
        super(GitSelector, self).__init__(main_window)
        self.settings.set_default('check_for_updates_on_start', True)
        self.settings.set_default('check_for_git_updates', False)
        self.packages = {
            'HyperSpy': [True, 'https://github.com/hyperspy/hyperspy'],
            'HyperSpyUI': [True, 'https://github.com/hyperspy/hyperspyui'],
            }
        self.ui.load_complete.connect(self._on_load_complete)
        if got_git:
            git.Git.GIT_PYTHON_GIT_EXECUTABLE = \
                    self.settings['git_executable'] or ''

    def create_actions(self):
        self.add_action(
            self.name + '.show_dialog', self.name, self.show_dialog,
            tip="Open dialog to select branch/version of HyperSpy/HyperSpyUI",)
        self.add_action(
            self.name + '.update_check', "Check for updates",
            self.update_check,
            tip="Check for new versions of HyperSpy/HyperSpyUI",)

    def _check_git(self):
        for package_name in self.packages.keys():
            if check_git_repo(package_name.lower()):
                git_ok = use_git
                if git_ok:
                    git_ok = check_git_cmd(True, self.ui)
                    self.settings['git_executable'] = \
                        git.Git.GIT_PYTHON_GIT_EXECUTABLE
                self.packages[package_name][0] = git_ok

    def create_menu(self):
        self.add_menuitem('Settings',
                          self.ui.actions[self.name + '.show_dialog'])
        self.add_menuitem('Settings',
                          self.ui.actions[self.name + '.update_check'])

    def _on_load_complete(self):
        if self.settings['check_for_updates_on_start', bool]:
            self.update_check(silent=True)

    def _perform_update(self, package):
        stream = VisualLogStream(self.ui)
        try:
            checkout_branch(package, stream)
        finally:
            diag = getattr(stream, 'dialog', None)
            if diag is not None:
                diag.btn_close.setEnabled(True)

    def update_check(self, silent=False):
        """
        Checks for updates to hyperspy and hyperspyUI.

        If the packages are not source installs, it checks for a new version on
        PyPI.

        Parameters
        ----------
        silent: bool
            If not silent (default), a message box will appear if no
            updates are available, with a message to that fact.

        Returns
        -------
        None.

        """
        self._check_git()
        available = {}
        for Name, (enabled, url) in self.packages.items():
            name = Name.lower()
            if enabled:
                if (check_git_repo(name) and
                        self.settings['check_for_git_updates', bool]):
                    # TODO: Check for commits to pull
                    pass
                else:
                    import xmlrpc.client
                    pypi = xmlrpc.client.ServerProxy(
                        'https://pypi.python.org/pypi')
                    found = pypi.package_releases(name)
                    if not found:
                        # Try to capitalize pkg name
                        if name == 'hyperspyui':
                            found = pypi.package_releases('hyperspyUI', True)
                        else:
                            found = pypi.package_releases(Name, True)
                    if found:
                        import pip
                        dist = [d for d in pip.get_installed_distributions()
                                if d.project_name.lower() == name]
                        if dist[0].version < found[0]:
                            available[name] = found[0]

        if available:
            w = self._get_update_list(available.keys())
            diag = self.ui.show_okcancel_dialog("Updates available", w)
            if diag.result() == QDialog.Accepted:
                for chk in w.children():
                    if isinstance(chk, QtWidgets.QCheckBox):
                        name = chk.text()
                        if available[name]:
                            name += '==' + available[name]
                        self._perform_update(name)
        elif not silent:
            mb = QMessageBox(QMessageBox.Information, tr("No updates"),
                             tr("No new updates were found."),
                             parent=self.ui)
            mb.exec_()

    def _get_update_list(self, names):
        w = QWidget()
        vbox = QVBoxLayout()
        vbox.addWidget(QLabel(tr("The following updates are available. Do you "
                                 "want to update them?")))
        for n in names:
            vbox.addWidget(QCheckBox(n))
        w.setLayout(vbox)
        return w

    def show_dialog(self):
        if len(self.dialogs) > 0:
            diag = self.dialogs[0]
        else:
            self._check_git()
            diag = VersionSelectionDialog(self, self.ui)
        self.open_dialog(diag)


class VersionSelectionDialog(QDialog):

    def __init__(self, plugin, parent=None):
        super(VersionSelectionDialog, self).__init__(parent)
        self.plugin = plugin
        self.packages = plugin.packages
        self._prev_indices = {}
        self.create_controls()

    def _cbo_changed(self, cbo, index):
        branch = cbo.itemData(index)
        stream = VisualLogStream(self.plugin.ui)
        try:
            checkout_branch(branch, stream)
        except ValueError:
            with block_signals(cbo):
                cbo.setCurrentIndex(self._prev_indices[cbo])
        finally:
            diag = getattr(stream, 'dialog', None)
            if diag is not None:
                diag.btn_close.setEnabled(True)
        self._prev_indices[cbo] = index

    def create_controls(self):
        self.setWindowTitle(tr(self.plugin.name))

        vbox = QVBoxLayout()
        form = QFormLayout()
        for Name, (enabled, url) in self.packages.items():
            name = Name.lower()
            cbo = QComboBox()
            if enabled:
                branches = get_branches(name, url)
                for n, b in branches.items():
                    cbo.addItem(n, b)
                if not check_git_repo(name):
                    cbo.insertItem(0, "<Select to change>", None)
                cbo.setCurrentIndex(0)
                self._prev_indices[cbo] = 0
                cbo.currentIndexChanged.connect(
                    partial(self._cbo_changed, cbo))
            else:
                cbo.setEditText("<git repository>")
                cbo.setToolTip(tr(
                    "This is installed in a git repository but we're set to "
                    "not use git."))
            cbo.setEnabled(enabled)
            form.addRow(Name + ':', cbo)

        vbox.addLayout(form)
        vbox.addWidget(QLabel(tr(
            "You should restart the application if you make any changes!")))

        btns = QDialogButtonBox(QDialogButtonBox.Ok, Qt.Horizontal, self)
        btns.accepted.connect(self.accept)
        vbox.addWidget(btns)
        self.setLayout(vbox)


class VisualLogStream(StringIO):
    def __init__(self, parent=None, buf=''):
        StringIO.__init__(self, buf)
        self.parent = parent
        self.dialog = None
        self.ignore = ['\x1b[?25l', '\x1b[?25h', '']
        self.clearln = '\r\x1b[K'

    def _ensure_dialog(self):
        if self.dialog is None:
            self.dialog = PipOutput(self.parent)
            self.dialog.setModal(True)
            self.dialog.show()

    def isatty(self):
        if self.closed:
            raise ValueError("Cannot call isatty when stream is closed.")
        return True

    def write(self, s):
        if s in self.ignore:
            return
        if s == self.clearln:
            v = self.getvalue()
            pos = v.rfind('\n', 0, len(v)-1)
            if pos > 0:
                self.truncate(pos+1)
                self.seek(pos+1)
        else:
            StringIO.write(self, s)
            self._ensure_dialog()
            self.dialog.output.setPlainText(self.getvalue())
            QtWidgets.QApplication.processEvents()


class PipOutput(QDialog):

    def __init__(self, parent=None):
        super(PipOutput, self).__init__(parent)
        self.create_controls()

    def sizeHint(self):
        def_sz = super(PipOutput, self).sizeHint()
        def_sz.setWidth(500)
        return def_sz

    def create_controls(self):
        self.setWindowTitle(tr("Updating via pip"))

        wFlags = self.windowFlags()
        if Qt.WindowCloseButtonHint == (wFlags & Qt.WindowCloseButtonHint):
            wFlags ^= Qt.WindowCloseButtonHint
            self.setWindowFlags(wFlags)
        vbox = QVBoxLayout()
        self.output = QTextEdit()
        if self.parent() and hasattr(self.parent(), 'console'):
            self.output.setFont(self.parent().console.font)
        vbox.addWidget(self.output)
        hbox = QHBoxLayout()
        hbox.addStretch()
        self.btn_close = QPushButton(tr("Close"))
        self.btn_close.clicked.connect(self.accept)
        self.btn_close.setEnabled(False)
        hbox.addWidget(self.btn_close)
        vbox.addLayout(hbox)
        self.setLayout(vbox)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Thu Jul 30 11:35:49 2015

@author: Vidar Tonaas Fauske
"""

import os

from hyperspyui.plugins.plugin import Plugin
from hyperspy.roi import BaseInteractiveROI
from hyperspyui.tools import LineTool


class LineMeasure(Plugin):
    name = "LineMeasure"

    def create_tools(self):
        self.tool = LineMeasureTool()
        self.tool.accepted[BaseInteractiveROI].connect(self.measure)
        self.tool.updated[BaseInteractiveROI].connect(self._continuous_update)
        self.add_tool(self.tool, self.ui.select_signal)

    def measure(self, roi):
        if self.tool.ndim == 2:
            self.ui.set_status("Line length: %G" % roi.length)

            s = self.tool._get_signal(self.tool.widget.ax.figure)
            roi(s).plot()
        else:
            print(roi)

    def _continuous_update(self, roi):
        if self.tool.ndim == 2:
            self.ui.set_status("Line length: %G" % roi.length)


class LineMeasureTool(LineTool):

    """
    Tool to crop signal interactively. Simply click and drag in a figure to
    create an ROI, and then press enter to apply cropping operation, or ESC to
    abort cropping. The cropping can also be aborted simply by selecting a
    different tool.
    """

    def get_name(self):
        return "Line measure"

    def get_icon(self):
        return os.path.dirname(__file__) + '/../images/ruler.svg'



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Tue Nov 04 16:32:55 2014

@author: Vidar Tonaas Fauske
"""

import os

from qtpy import QtGui, QtCore, QtSvg, QtWidgets
from qtpy.QtCore import Qt, Signal

from functools import partial
import traits.api as t
import traitsui.api as tu

from hyperspyui.util import create_add_component_actions, win2sig


def tr(text):
    return QtCore.QCoreApplication.translate("DataViewWidget", text)


NameCol = 1
VisibilityCol = 0


class ComponentEditorHandler(tu.Handler):

    def setattr(self, info, object, name, value):
        # Set the value etc.
        tu.Handler.setattr(self, info, object, name, value)
        if name in ('value', 'std'):
            try:
                # Make sure the value is actually stored in array
                object.store_current_value_in_array()
            except AttributeError:
                pass


class VisbilityDelegate(QtWidgets.QStyledItemDelegate):

    def __init__(self, parent=None, icons=None):
        if icons is None or len(icons) < 1:
            icons = []
            path = os.path.dirname(__file__)
            for fn in [path + "/../images/visibility_on.svg",
                       path + "/../images/visibility_off.svg"]:
                renderer = QtSvg.QSvgRenderer(fn)
                pm = QtGui.QPixmap(12, 12)
                pm.fill(Qt.transparent)
                painter = QtGui.QPainter(pm)
                renderer.render(painter)
                icons.append(pm)
            icons = tuple(icons)
        elif len(icons) < 2:
            icons = tuple(icons) + (QtGui.QPixmap(),)
        else:
            icons = tuple(icons)
        self.icons = icons
        self.margin = 2
        super(VisbilityDelegate, self).__init__(parent)

    def iconPos(self, icon, option):
        return QtCore.QPoint(option.rect.right() - icon.width() - self.margin,
                             option.rect.center().y() - icon.height()/2)

    def sizeHint(self, option, index):
        size = super(VisbilityDelegate, self).sizeHint(option, index)

        # Make sure we have room for the icon
        if Qt.Checked == index.data(Qt.CheckStateRole):
            icon = self.icons[0]
        else:
            icon = self.icons[1]
        size.setWidth(max(size.width(), + icon.width() + self.margin * 2))
        size.setHeight(max(size.height(), icon.height() + self.margin * 2))
        return size

    def paint(self, painter, option, index):
        if option.state & QtWidgets.QStyle.State_Selected:
            painter.fillRect(option.rect, option.palette.highlight())
        if Qt.Checked == index.data(Qt.CheckStateRole):
            icon = self.icons[0]
        elif Qt.Unchecked == index.data(Qt.CheckStateRole):
            icon = self.icons[1]
        else:
            return
        painter.drawPixmap(self.iconPos(icon, option), icon)


class HyperspyItem(QtWidgets.QTreeWidgetItem):

    def __init__(self, parent, itemtype, hspy_item):
        super(HyperspyItem, self).__init__(parent, itemtype)
        self.hspy_item = hspy_item
        self.setFlags(Qt.ItemIsSelectable | Qt.ItemIsUserCheckable |
                Qt.ItemIsEnabled)
        self.setText(NameCol, hspy_item.name)

    def data(self, column, role):
        if column == NameCol:
            if role == Qt.UserRole:
                return self.hspy_item
        elif column == VisibilityCol:
            if role == Qt.CheckStateRole:
                # Asking about visibility
                # Only return valid state for SignalType
                if self.type() == DataViewWidget.SignalType:
                    s = self.hspy_item
                    vis = False
                    for w in [s.navigator_plot, s.signal_plot]:
                        if w is not None and not w.isMinimized():
                            vis = True
                    if vis:
                        return Qt.Checked
                    else:
                        return Qt.Unchecked
                else:
                    return None
        return super(HyperspyItem, self).data(column, role)

    def setData(self, column, role, value):
        if column == VisibilityCol and role == Qt.CheckStateRole \
                and self.type() == DataViewWidget.SignalType:
            self._on_toggle_visibility()
        else:
            super(HyperspyItem, self).setData(column, role, value)

    def _on_toggle_visibility(self):
        checked = Qt.Checked == self.data(VisibilityCol, Qt.CheckStateRole)
        s = self.hspy_item
        figs = [p for p in (s.signal_plot, s.navigator_plot) if p is not None]
        if len(figs) < 1:
            return
        for p in figs:
            if checked:
                p.showMinimized()
            else:
                if p.isMinimized():
                    p.showNormal()
                else:
                    self.treeWidget().main_window.main_frame.\
                        setActiveSubWindow(p)


class DataViewWidget(QtWidgets.QWidget):

    """
    A custom QTreeWidget, that handles the Signal-Model-Component hierarchy.
    The relationships are displayed in a tree structure, and helps keep track
    of the relationships between them. Also makes handeling several Models per
    Signal easier.
    """

    # Enums
    SignalType = QtWidgets.QTreeWidgetItem.UserType
    ModelType = QtWidgets.QTreeWidgetItem.UserType + 1
    ComponentType = QtWidgets.QTreeWidgetItem.UserType + 2

    def __init__(self, main_window, parent=None):
        super(DataViewWidget, self).__init__(parent)
        self.main_window = main_window
        self.tree = QtWidgets.QTreeWidget(self)
        self.tree.header().close()
        self.lut = {}
        self.editor_visible = True
        self.tree.setColumnCount(2)
        self.editor = QtWidgets.QVBoxLayout()
        self.editor_bottom_padding = 20

        vbox = QtWidgets.QVBoxLayout()
        vbox.addWidget(self.tree)
        vbox.addLayout(self.editor)
        vbox.setContentsMargins(0, 0, 0, 0)
        self.setLayout(vbox)
        sp = self.sizePolicy()
        sp.setVerticalPolicy(QtWidgets.QSizePolicy.Expanding)
        self.setSizePolicy(sp)
        sp = self.tree.sizePolicy()
        sp.setVerticalPolicy(QtWidgets.QSizePolicy.Expanding)
        self.tree.setSizePolicy(sp)

        self.tree.setContextMenuPolicy(Qt.CustomContextMenu)
        self.tree.customContextMenuRequested[QtCore.QPoint].connect(self.onCustomContextMenu)
        self.tree.currentItemChanged.connect(self.currentItemChanged)
        self.tree.itemDoubleClicked.connect(self.itemDoubleClicked)

        self.tree.setItemDelegateForColumn(VisibilityCol,
                                           VisbilityDelegate(self.tree))
        self.tree.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)

    def _add(self, item, itemtype, parent=None):
        """
        Make a QTreeWidgetItem for data item, and insert it below parent.
        The parent can be either a data item, or a QTreeWidgetItem.
        """
        if parent is None:
            parent = self.tree
        elif not isinstance(parent, QtWidgets.QTreeWidgetItem):
            parent = self.lut[parent]
        twi = HyperspyItem(parent, itemtype, item)
        if itemtype == self.SignalType:
            self.tree.resizeColumnToContents(VisibilityCol)
        twi.setExpanded(True)
        self.lut[item] = twi
        if parent is self.tree:
            self.tree.addTopLevelItem(twi)
        else:
            parent.addChild(twi)
        return twi

    def clear_editor(self):
        item = self.editor.takeAt(0)
        while item:
            w = item.widget()
            if w:
                w.close()
            item = self.editor.takeAt(0)

    def set_traits_editor(self, traits_dialog):
        self.clear_editor()
        sp = traits_dialog.sizePolicy()
        sp.setVerticalPolicy(QtWidgets.QSizePolicy.Fixed)
        traits_dialog.setSizePolicy(sp)
        self.editor.addWidget(traits_dialog)
        self.editor.addSpacing(self.editor_bottom_padding)
        traits_dialog.show()

    def currentItemChanged(self, current, previous):
        if self.editor_visible:
            if current and current.type() == self.ComponentType:
                comp = current.data(NameCol, Qt.UserRole)
                if isinstance(comp, t.HasTraits):
                    self.main_window.capture_traits_dialog(
                        self.set_traits_editor)
                    self.edit_traits(comp, False)
            else:
                self.clear_editor()

    def itemDoubleClicked(self, item, column):
        if column == VisibilityCol:
            return      # Don't count double clicks on check
        if item.type() == self.ComponentType:
            item = item.parent().parent()
        elif item.type() == self.ModelType:
            item = item.parent()
        val = Qt.Checked == item.data(VisibilityCol, Qt.CheckStateRole)
        state = Qt.Checked if not val else Qt.Unchecked
        item.setData(VisibilityCol, Qt.CheckStateRole, state)

    def edit_traits(self, comp, buttons=True):
        try:
            items = [tu.Item('name'), tu.Item('active')]
            for p in comp.parameters:
                name = '.'.join(('object', p.name))
                p_label = p.name.replace('_', ' ').capitalize()
                vi = tu.Item(name + '.value', label=p_label,
                             editor=tu.RangeEditor(low_name=name + '.bmin',
                                                   high_name=name + '.bmax'))
                items.extend((vi, tu.Item(name + '.free')))
            view = tu.View(*items,
                           handler=ComponentEditorHandler(),
                           buttons=tu.OKCancelButtons if buttons else [],
                           default_button=tu.OKButton,
                           kind='live',
                           resizable=True)
            comp.edit_traits(view=view)
        except AttributeError:
            pass

    def onCustomContextMenu(self, point):
        """
        Displays the context menu for whatever is under the supplied point.
        """
        item = self.tree.itemAt(point)
        if not item:
            return
        cm = QtWidgets.QMenu(self.tree)
        if item.type() == self.SignalType:
            sig = item.data(NameCol, Qt.UserRole)

            # Add all actions defined on object
            for ac in sig.actions.values():
                cm.addAction(ac)

        elif item.type() == self.ModelType:
            model = item.data(NameCol, Qt.UserRole)

            # Add all actions defined on object
            for ac in model.actions.values():
                cm.addAction(ac)

            # Add "add component" actions
            cm.addSeparator()
            comp_actions = create_add_component_actions(self.tree,
                                                        model.add_component,
                                                        prefix="Add ")
            for ac in comp_actions.values():
                cm.addAction(ac)
        elif item.type() == self.ComponentType:
            comp = item.data(NameCol, Qt.UserRole)
            model = item.parent().data(NameCol, Qt.UserRole)

            # Fit action
            ac = QtWidgets.QAction(tr("&Fit component"), self.tree)
            f = partial(model.fit_component, comp)
            ac.triggered.connect(f)
            cm.addAction(ac)

            # Configure action
            ac = QtWidgets.QAction(tr("&Configure"), self.tree)
            f = partial(self.edit_traits, comp)
            ac.triggered.connect(f)
            cm.addAction(ac)

            cm.addSeparator()

            # Remove action
            ac = QtWidgets.QAction(tr("&Delete"), self.tree)
            f = partial(model.remove_component, comp)
            ac.triggered.connect(f)
            cm.addAction(ac)
        cm.exec_(self.tree.mapToGlobal(point))

    def keyPressEvent(self, event):
        citem = self.tree.currentItem()
        if event.key() == Qt.Key_Delete:
            data = citem.data(NameCol, Qt.UserRole)
            if citem.type() == self.SignalType:
                data.close()
            elif citem.type() == self.ModelType:
                data.actions['delete'].trigger()
            elif citem.type() == self.ComponentType:
                model = citem.parent().data(NameCol, Qt.UserRole)
                model.remove_component(data)
        else:
            super(DataViewWidget, self).keyPressEvent(event)

    def _remove(self, key):
        if key in self.lut:
            item = self.lut[key]
            if item.parent() is None:
                self.tree.takeTopLevelItem(self.tree.indexOfTopLevelItem(item))
            else:
                item.parent().removeChild(item)
            self.lut.pop(key)

    def add_signal(self, signal):
        item = self._add(signal, self.SignalType)
        signal.model_added[object].connect(self.add_model)
        signal.model_removed[object].connect(self.remove)

        for m in signal.models:
            self.add_model(m, signal)
        self.tree.setCurrentItem(item)

    def add_model(self, model, signal=None):
        if signal is None:
            signal = model.signal
        self._add(model, self.ModelType, parent=signal)
        model.added[object, object].connect(self.add_component)
        model.removed[object].connect(self.remove)
        for c in model.components.values():
            self.add_component(c, model)

    def add_component(self, component, model):
        ci = self._add(component, self.ComponentType,
                       parent=model)
        if isinstance(component, t.HasTraits):
            def update_name(new_name):
                ci.setText(NameCol, new_name)
            component.on_trait_change(update_name, 'name')

    def add(self, object, type, parent=None):
        self._add(object, type, parent)

    def remove(self, object):
        self._remove(object)

    def on_mdiwin_activated(self, mdi_figure):
        """
        Can be connected to an MdiArea's subWindowActivated signal to sync
        the selected signal.
        """
        sf = win2sig(mdi_figure, self.main_window.signals)
        for i in range(self.tree.topLevelItemCount()):
            item = self.tree.topLevelItem(i)
            s = item.data(NameCol, Qt.UserRole)
            # In case topLevelItems are not all SignalWrappers in future
            if s is sf and s is not self.get_selected_wrapper():
                self.tree.setCurrentItem(item)
                return

    def get_selected_wrappers(self):
        """
        Returns a list of all selected signals. Any selected Models or
        Components will select their Signal parent.
        """
        items = self.tree.selectedItems()
        signals = []
        for i in items:
            if i.type() == self.ComponentType:
                i = i.parent().parent()
            elif i.type() == self.ModelType:
                i = i.parent()
            if i.type() != self.SignalType:
                raise TypeError("Selection of wrong type")
            s = i.data(NameCol, Qt.UserRole)
            if s not in signals:
                signals.append(s)

        return signals

    def get_selected_wrapper(self):
        """
        Returns the first selected Signal. Any selected Models or Components
        will select their Signal parent.
        """
        items = self.tree.selectedItems()
        if len(items) < 1:
            return None
        item = items[0]
        if item.type() == self.ComponentType:
            item = item.parent().parent()
        elif item.type() == self.ModelType:
            item = item.parent()
        if item.type() != self.SignalType:
            raise TypeError("Selection of wrong type")
        return item.data(NameCol, Qt.UserRole)

    def get_selected_model(self):
        """
        Returns the first selected Model. Any selected Signals/Components
        will select their Model child/parent.
        """
        items = self.tree.selectedItems()
        if len(items) < 1:
            return None
        item = items[0]
        if item.type() == self.ComponentType:
            item = item.parent()
        elif item.type() == self.SignalType:
            if item.childCount() < 1:
                return None
            item = item.child(0)
        if item.type() != self.ModelType:
            raise TypeError("Selection of wrong type")
        return item.data(NameCol, Qt.UserRole)

    def get_selected_component(self):
        raise NotImplementedError()



from hyperspyui.plugins.plugin import Plugin

from hyperspy.utils.plot import plot_images, plot_spectra


class PlotUtils(Plugin):
    name = "Plot utils"

    def create_actions(self):
        self.add_action(self.name + '.plot_images', "Plot images",
                        self.plot_images,
                        tip="Plots several images together in one figure.")
        self.add_action(self.name + '.plot_spectra', "Plot spectra",
                        self.plot_spectra,
                        tip="Plots several spectra together in one figure.")

    def create_menu(self):
        self.add_menuitem('Image',
                          self.ui.actions[self.name + '.plot_images'])
        self.add_menuitem('Spectrum',
                          self.ui.actions[self.name + '.plot_spectra'])

    def plot_images(self, images=None):
        if images is None:
            images = self.ui.get_selected_signals()
        images_t = []
        for im in images:
            if len(im.axes_manager.shape) != 2 and len(images) > 1:
                raise ValueError(
                    "Signal shape invalid for plot_images(): %s" % str(im))
            if im.axes_manager.signal_dimension == 2:
                images_t.append(im)
            elif len(im.axes_manager.shape) != 2:
                raise ValueError(
                    "Signal needs to be an image, or a stack of images")
            else:
                images_t.append(im.as_signal2D((0, 1)))
        plot_images(images_t, colorbar=None, axes_decor=None)

    def plot_spectra(self, spectra=None):
        if spectra is None:
            spectra = self.ui.get_selected_signals()
            if len(spectra) == 1:
                spectra = spectra[0]

        spectra_t = []
        for im in spectra:
            if len(im.axes_manager.shape) != 1 and len(spectra) > 1:
                raise ValueError(
                    "Signal shape invalid for plot_spectra(): %s" % str(im))
            if im.axes_manager.signal_dimension == 1:
                spectra_t.append(im)
            elif len(im.axes_manager.shape) != 1:
                raise ValueError(
                    "Signal needs to be 1-dimensional, or a stack of 1D signals")
            else:
                spectra_t.append(im.as_signal1D(0))

        plot_spectra(spectra_t)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Sat Feb 21 12:04:06 2015

@author: Vidar Tonaas Fauske
"""


"""
Utility to make it easier to create plugins directly through the UI.
"""

import os
import string
import hyperspyui.plugins.plugin

header = """from hyperspyui.plugins.plugin import Plugin
import numpy as np
import hyperspy.api as hs

class {0}(Plugin):
    name = "{1}"

    def create_actions(self):"""

action_noicon = """
        self.add_action(self.name + '.default', self.name, self.default,
                        tip="")
"""

action_icon = """
        self.add_action(self.name + '.default', self.name, self.default,
                        icon="{0}",
                        tip="")
"""

menu_def = """
    def create_menu(self):
        self.add_menuitem('{0}', self.ui.actions[self.name + '.default'])
"""

toolbar_def = """
    def create_toolbars(self):
        self.add_toolbar_button('{0}', self.ui.actions[self.name + '.default'])
"""

default = """
    def default(self):
{0}
"""


def indent(lines, amount, ch=' '):
    padding = amount * ch
    return padding + ('\n' + padding).join(lines.split('\n'))


def suggest_plugin_filename(name):
    filename = name.lower() + '.py'
    dirname = os.path.dirname(hyperspyui.plugins.plugin.__file__)
    path = dirname + os.path.sep + filename
    return path


def create_plugin_code(code, name, category=None, menu=False, toolbar=False,
                       icon=None):
    """Create a plugin with an action that will execute 'code' when triggered.
    If 'menu' and/or 'toolbar' is True, the corresponding items will be added
    for the action.
    """

    if category is None:
        category = name

    safe_name = string.capwords(name).replace(" ", "")
    plugin_code = header.format(safe_name, name)
    if icon is None:
        plugin_code += action_noicon.format()
    else:
        plugin_code += action_icon.format(icon)
    if menu:
        plugin_code += menu_def.format(category)
    if toolbar:
        plugin_code += toolbar_def.format(category)

    # Indent code by two levels
    code = indent(code, 2 * 4)
    plugin_code += default.format(code)
    try:
        import autopep8
        plugin_code = autopep8.fix_code(plugin_code,
                                        options=autopep8.parse_args(
                                         ['--aggressive', '--aggressive', '']))
    except ImportError:
        pass
    return plugin_code


def create_plugin_file(code, name, category=None, menu=False, toolbar=False,
                       filename=None):
    """Create a plugin with an action that will execute 'code' when triggered.
    If 'menu' and/or 'toolbar' is True, the corresponding items will be added
    for the action.
    """
    if filename is None:
        path = suggest_plugin_filename(name)
    elif os.path.isabs(filename):
        path = filename
    else:
        dirname = os.path.dirname(hyperspyui.plugins.plugin.__file__)
        path = dirname + os.path.sep + filename

    with open(path, 'w') as f:
        f.write(create_plugin_code(code, name, category, menu, toolbar))
    return path



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Fri Oct 24 18:27:15 2014

@author: Vidar Tonaas Fauske
"""

from .util import fig2win
from qtpy import QtCore

from .modelwrapper import ModelWrapper
from .actionable import Actionable


class SignalWrapper(Actionable):
    closing = QtCore.Signal()
    model_added = QtCore.Signal(object)
    model_removed = QtCore.Signal(object)

    _untitled_counter = 0

    def __init__(self, signal, mainwindow, name=None):
        super(SignalWrapper, self).__init__()
        self.signal = signal
        # Override replot on Signal instance
        self._old_replot = signal._replot
        signal._replot = self._replot
        if name is None:
            if signal.metadata.General.title:
                name = signal.metadata.General.title
            elif signal.tmp_parameters.has_item('filename'):
                name = signal.tmp_parameters.filename
            else:
                name = "Untitled %d" % SignalWrapper._untitled_counter
                SignalWrapper._untitled_counter += 1
        self.name = name
        self.figures = []
        self.mainwindow = mainwindow
        self.models = []

        self._keep_on_close = 0
        self._magic_jump = (8, 30)

        self.navigator_plot = None
        self.signal_plot = None

        self._nav_geom = None
        self._sig_geom = None
        self._replotargs = ((), {})

        self._model_id = 1

        self.add_action('plot', "&Plot", self.replot)
        self.add_action('add_model', "Add &model", self.make_model)
        self.add_separator()
        self.add_action('close', "&Close", self.close)

    @property
    def keep_on_close(self):
        return self._keep_on_close > 0

    @keep_on_close.setter
    def keep_on_close(self, value):
        if value:
            self._keep_on_close += 1
        else:
            if self._keep_on_close > 0:
                self._keep_on_close -= 1

    def plot(self, *args, **kwargs):
        self.keep_on_close = True
        self.signal.plot(*args, **kwargs)
        self.keep_on_close = False
        self.update_figures()
        self._replotargs = (args, kwargs)
        self.mainwindow.main_frame.subWindowActivated.emit(
            self.mainwindow.main_frame.activeSubWindow())
        # Redraw plot needed for pyqt5
        if self.signal._plot and self.signal._plot.signal_plot:
            self.signal._plot.signal_plot.figure.canvas.draw_idle()

    def _replot(self):
        if self.signal._plot is not None:
            if self.signal._plot.is_active:
                self.replot()

    def replot(self):
        old = self.mainwindow.updatesEnabled()
        self.mainwindow.setUpdatesEnabled(False)
        try:
            self.plot(*self._replotargs[0], **self._replotargs[1])
        finally:
            self.mainwindow.setUpdatesEnabled(old)

    def switch_signal(self, new_signal):
        """
        Switch the signal wrapped by this wrapper. To complete the switch, the
        signal should also be replotted if previously plotted. For performance
        reasons this is left as the responsibility of the caller.
        """
        old_signal = self.signal
        self.signal = new_signal
        idx = -1
        for i, s in enumerate(self.mainwindow.hspy_signals):
            if s is old_signal:
                idx = i
                break
        self.mainwindow.lut_signalwrapper[new_signal] = self
        del self.mainwindow.lut_signalwrapper[old_signal]
        if idx >= 0:
            self.mainwindow.hspy_signals[idx] = new_signal

    def update(self):
        if self.navigator_plot is not None:
            self.navigator_plot.update()
        if self.signal_plot is not None:
            self.signal_plot.update()

    def update_figures(self):
        old_nav = self.navigator_plot
        old_sig = self.signal_plot
        self.remove_figure(old_nav)
        self.remove_figure(old_sig)
        self.navigator_plot = None
        self.signal_plot = None

        atleast_one_changed = False

        # If we have a navigator plot
        if self.signal._plot and self.signal._plot.navigator_plot:
            # Set internal `navigator_plot` to window containing it
            navi = self.signal._plot.navigator_plot.figure
            self.navigator_plot = fig2win(navi, self.mainwindow.figures)
            # Did the window change?
            if old_nav is not self.navigator_plot:
                # Process the plot
                title = navi.axes[0].set_title("")  # remove title
                title.set_visible(False)
                # Wire closing event
                self.navigator_plot.closing.connect(self.nav_closing)
                # Set a reference on window to self
                self.navigator_plot.setProperty('hyperspyUI.SignalWrapper',
                                                self)
                # Add to figures list
                self.add_figure(self.navigator_plot)

                # Did we have a previous window?
                if old_nav is not None:
                    navi.tight_layout()
                    # Save geometry of old, and make sure it is closed
                    self._nav_geom = old_nav.saveGeometry()
                    old_nav.closing.disconnect(self.nav_closing)
                    old_nav.close()
                    atleast_one_changed = True
                # If we have stored geometry, and a valid plot, restore
                if self._nav_geom is not None and self.navigator_plot is not None:
                    self.navigator_plot.restoreGeometry(self._nav_geom)
                self._nav_geom = None

        if self.signal._plot and self.signal._plot.signal_plot is not None:
            sigp = self.signal._plot.signal_plot.figure
            self.signal_plot = fig2win(sigp, self.mainwindow.figures)
            if old_sig is not self.signal_plot:
                title = sigp.axes[0].set_title("")
                title.set_visible(False)
                self.signal_plot.closing.connect(self.sig_closing)
                self.signal_plot.setProperty('hyperspyUI.SignalWrapper', self)
                self.add_figure(self.signal_plot)
                if old_sig is not None:
                    sigp.tight_layout()
                    self._sig_geom = old_sig.saveGeometry()
                    old_sig.closing.disconnect(self.sig_closing)
                    old_sig.close()
                    atleast_one_changed = True
                if self._sig_geom is not None and self.signal_plot is not None:
                    self.signal_plot.restoreGeometry(self._sig_geom)
                self._sig_geom = None

        if atleast_one_changed:
            self.mainwindow.check_action_selections()

    def add_figure(self, fig):
        self.figures.append(fig)

    def remove_figure(self, fig):
        if fig in self.figures:
            self.figures.remove(fig)

    def as_signal2D(self, axis=(0, 1)):
        signal = self.signal
        self.close()  # Store geomtery and close
        # Swap geometries
        tmp = self._sig_geom
        self._sig_geom = self._nav_geom
        self._nav_geom = tmp
        self.signal = signal.as_signal2D(axis)

    def as_signal1D(self, axis=0):
        signal = self.signal
        self.close()  # Store geomtery and close
        # Swap geometries
        tmp = self._sig_geom
        self._sig_geom = self._nav_geom
        self._nav_geom = tmp
        self.signal = signal.as_signal1D(axis)

    def make_model(self, *args, **kwargs):
        m = self.signal.create_model(*args, **kwargs)
        self.mainwindow.record_code("signal = ui.get_selected_signal()")
        self.mainwindow.record_code("model = signal.create_model()")
#        modelname = self.signal.metadata.General.title
        modelname = "Model %d" % self._model_id
        self._model_id += 1
        mw = ModelWrapper(m, self, modelname)
        self.add_model(mw)
        mw.plot()
        return mw

    def add_model(self, model):
        self.models.append(model)
        self.model_added.emit(model)

    def remove_model(self, model):
        self.models.remove(model)
        self.model_removed.emit(model)
        self.plot()

    def nav_closing(self):
        if self.navigator_plot:
            p = self.navigator_plot.pos()
            self.navigator_plot.move(p.x() + self._magic_jump[0],
                                     p.y() + self._magic_jump[1])
            self._nav_geom = self.navigator_plot.saveGeometry()
            self.navigator_plot = None
        if self.signal_plot is None:
            self._closed()

    def sig_closing(self):
        if self.signal_plot:
            p = self.signal_plot.pos()
            # For some reason the position changes -8,-30 on closing, at least
            # it does on windows 7, Qt4.
            self.signal_plot.move(p.x() + self._magic_jump[0],
                                  p.y() + self._magic_jump[1])
            self._sig_geom = self.signal_plot.saveGeometry()
        if self.navigator_plot is not None:
            self.navigator_plot.close()
            self.navigator_plot = None
        self.signal_plot = None
        self._closed()

    def close(self):
        if self.signal_plot is not None:
            self.signal_plot.close()
            self.signal_plot = None

        if self.navigator_plot is not None:
            self.navigator_plot.close()
            self.navigator_plot = None
        self._closed()

    def _closed(self):
        if not self.keep_on_close:
            self.closing.emit()
        # TODO: Should probably be with by events for concistency
        if self in self.mainwindow.signals and not self.keep_on_close:
            self.mainwindow.signals.remove(self)
            self.signal._replot = self._old_replot
            self._old_replot = None
            self.signal = None



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.
"""
Created on Wed Jan 21 14:49:08 2015

@author: Vidar Tonaas Fauske
"""


from qtpy import QtCore
from qtpy.QtWidgets import (QCheckBox, QPushButton, QHBoxLayout, QVBoxLayout, 
                            QFormLayout, QDoubleSpinBox, QGroupBox, QSpinBox,
                            QRadioButton)

from hyperspyui.plugins.plugin import Plugin
from hyperspyui.widgets.extendedqwidgets import ExToolWindow
from scipy.ndimage import rotate
import numpy as np
from hyperspyui.util import win2sig, dummy_context_manager
from hyperspyui.signalwrapper import SignalWrapper


def tr(text):
    return QtCore.QCoreApplication.translate("ImageRotation", text)


class ImageRotation_Plugin(Plugin):
    name = 'Image Rotation'

    def create_actions(self):
        self.settings.set_enum_hint('new_or_replace', ['new', 'replace'])
        self.add_action('rotate', "Rotate", self.show_rotate_dialog,
                        icon='rotate.svg',
                        tip="Rotate an image",
                        selection_callback=self.selection_rules)

    def create_menu(self):
        self.add_menuitem('Image', self.ui.actions['rotate'])

    def create_toolbars(self):
        self.add_toolbar_button("Image", self.ui.actions['rotate'])

    def selection_rules(self, win, action):
        """
        Callback to determine if rotation is valid for the passed window.
        """
        s = win2sig(win, self.ui.signals)
        ok = False
        if s is not None:
            if win == s.navigator_plot and \
                    s.signal.axes_manager.navigation_dimension >= 2:
                ok = True
            elif win == s.signal_plot and \
                    s.signal.axes_manager.signal_dimension == 2:
                ok = True
        if ok:
            action.setEnabled(True)
        else:
            action.setEnabled(False)

    def rotate_signal(self, angle, signal=None, reshape=False, out=None,
                      record=True, *args, **kwargs):
        if signal is None:
            signal, axes, _ = self.ui.get_selected_plot()
            if isinstance(axes, str):
                axm = signal.signal.axes_manager
                if axes.startswith("nav"):
                    axes = (axm._axes.index(axm.navigation_axes[0]),
                            axm._axes.index(axm.navigation_axes[1]))
                elif axes.startswith("sig"):
                    axes = (axm._axes.index(axm.signal_axes[0]),
                            axm._axes.index(axm.signal_axes[1]))
            kwargs['axes'] = axes
        if isinstance(signal, SignalWrapper):
            signal = signal.signal
        if record:
            self.record_code(
                r"<p>.rotate_signal({0}, reshape={1}, {2}, {3})".format(
                    angle, reshape, args, kwargs))
        # Reframe into 0-360 deg
        angle %= 360
        const_mode = 'mode' not in kwargs or kwargs['mode'] == 'constant'
        # Check if multiple of 90 deg -> Fast rotation
        if round(angle % 90, 2) == 0 and (reshape or const_mode):
            angle = 90 * (angle // 90)
            axes = kwargs.pop('axes', (1, 0))
            k = angle // 90
            if k > 0:
                if k == 1:  # 90 deg
                    # Invert axes[0]
                    s = (slice(None),)* axes[0] + (slice(None, None, -1),) + \
                        (Ellipsis,)
                elif k == 2:  # 180 deg
                    # Invert both axes
                    axisA = min(axes[0], axes[1])
                    axisB = max(axes[0], axes[1])
                    adiff = (axisB-axisA-1)
                    s = (slice(None),)* axisA + (slice(None, None, -1),) + \
                        (slice(None),)* adiff + (slice(None, None, -1),) + \
                        (Ellipsis,)
                elif k == 3:  # 270 deg
                    # Invert axes[1]
                    s = (slice(None),)* axes[1] + (slice(None, None, -1),) + \
                        (Ellipsis,)
                data = signal.data[s]
            else:
                data = signal.data
            if k % 2 == 1:     # Rotating 90 or 270
                # TODO: Swap HyperSpy axes as well!
                data = np.swapaxes(data, axes[0], axes[1])
                if not reshape:
                    # By checking for constant mode before, padding is easy
                    cval = kwargs.pop('cval', 0.0)
                    bkgr = np.ones_like(signal.data) * cval
                    fa, la = min(axes), max(axes)
                    fd, ld = np.shape(data)[fa], np.shape(data)[la]
                    crop = np.abs(ld - fd) // 2
                    if fd < ld:  # Expand first, crop last
                        s = (slice(None),) * fa + (slice(0, fd),) + \
                            (slice(None),) * (la - fa - 1) + \
                            (slice(crop, crop + fd),) + (Ellipsis,)
                        t = (slice(None),) * fa + (slice(crop, crop + fd),) + \
                            (Ellipsis,)
                    else:       # Crop first, expand last
                        s = (slice(None),) * fa + (slice(crop, crop + ld),) + \
                            (slice(None),) * (la - fa - 1) + \
                            (slice(0, ld),) + (Ellipsis,)
                        t = (slice(None),) * la + (slice(crop, crop + ld),) + \
                            (Ellipsis,)
                    bkgr[t] = data[s]
                    data = bkgr
        else:
            data = rotate(signal.data, angle, reshape=reshape, *args, **kwargs)
        if out is None:
            sig = signal._deepcopy_with_new_data(None)
        else:
            sig = out
            old_shape = out.data.shape
        sig.data = data
        if out is None:
            if reshape:
                sig.get_dimensions_from_data()
                diff = np.array(sig.data.shape) - signal.data.shape
                for i, ax in enumerate(sig.axes_manager._axes):
                    dx = diff[i] * 0.5 * ax.scale
                    ax.offset -= dx
            return sig
        else:
            if sig.data.shape != old_shape:
                if hasattr(out.axes_manager, 'events') and hasattr(
                        out.axes_manager.events, 'any_axis_changed'):
                    cm = out.axes_manager.events.any_axis_changed.suppress
                else:
                    cm = dummy_context_manager
                with cm():
                    out.get_dimensions_from_data()
                    diff = np.array(out.data.shape) - old_shape
                    for i, ax in enumerate(out.axes_manager._axes):
                        dx = diff[i] * 0.5 * ax.scale
                        ax.offset -= dx
                # TODO: TAG: Functionality check
                if hasattr(out.axes_manager, 'events') and hasattr(
                        out.axes_manager.events, 'any_axis_changed'):
                    out.axes_manager.events.any_axis_changed.trigger(
                        out.axes_manager)
            # TODO: TAG: Functionality check
            if hasattr(out, 'events') and hasattr(out.events, 'data_changed'):
                out.events.data_changed.trigger(out)

    def on_dialog_accept(self):
        self.settings['angle'] = self.dialog.num_angle.value()
        if self.dialog.opt_new.isChecked():
            self.settings['new_or_replace'] = 'new'
        else:
            self.settings['new_or_replace'] = 'replace'
        self.settings['reshape'] = self.dialog.chk_reshape.isChecked()
        self.settings['preview'] = self.dialog.gbo_preview.isChecked()
        self.settings['grid'] = self.dialog.chk_grid.isChecked()
        self.settings['grid_spacing'] = self.dialog.num_grid.value()
        self.dialog.deleteLater()
        self.dialog = None

    def show_rotate_dialog(self):
        signal, space, _ = self.ui.get_selected_plot()
        if space not in ("navigation", "signal"):
            return
        self.dialog = ImageRotationDialog(signal, space, self.ui, self)
        if 'angle' in self.settings:
            self.dialog.num_angle.setValue(self.settings['angle', float])
        if 'new_or_replace' in self.settings:
            v = self.settings['new_or_replace']
            if v == 'new':
                self.dialog.opt_new.setChecked(True)
            elif v == 'replace':
                self.dialog.opt_replace.setChecked(True)
        if 'reshape' in self.settings:
            self.dialog.chk_reshape.setChecked(self.settings['reshape', bool])
        if 'preview' in self.settings:
            self.dialog.gbo_preview.setChecked(self.settings['preview', bool])
        if 'grid' in self.settings:
            self.dialog.chk_grid.setChecked(self.settings['grid', bool])
        if 'grid_spacing' in self.settings:
            self.dialog.num_grid.setValue(self.settings['grid_spacing', int])
        self.dialog.accepted.connect(self.on_dialog_accept)
        self.dialog.show()


class ImageRotationDialog(ExToolWindow):

    def __init__(self, signal, axes, parent, plugin):
        super(ImageRotationDialog, self).__init__(parent)
        self.ui = parent
        self.create_controls()
        self.accepted.connect(self.ok)
        self.rejected.connect(self.close_new)
        self.signal = signal
        self.plugin = plugin
        self.new_out = None
        self._connected_updates = False
        if isinstance(axes, str):
            axm = signal.signal.axes_manager
            if axes.startswith("nav"):
                axes = (axm._axes.index(axm.navigation_axes[0]),
                        axm._axes.index(axm.navigation_axes[1]))
            elif axes.startswith("sig"):
                axes = (axm._axes.index(axm.signal_axes[0]),
                        axm._axes.index(axm.signal_axes[1]))
        self.axes = axes
        self.setWindowTitle(tr("Rotate"))

        # TODO: TAG: Functionality check
        if not hasattr(signal.signal, 'events'):
            self.gbo_preview.setVisible(False)

        # TODO: Add dynamic rotation, e.g. one that rotates when source
        # signal's data_changed event triggers

    def connect(self):
        # TODO: Don't have to con/dis those in gbo
        self.opt_new.toggled.connect(self.close_new)
        self.num_angle.valueChanged.connect(self.update)
        self.chk_grid.toggled.connect(self.update)
        self.num_grid.valueChanged.connect(self.update)
        self.chk_reshape.toggled.connect(self.update)
        self.opt_new.toggled.connect(self.update)
        self.opt_replace.toggled.connect(self.update)

    def disconnect(self):
        self.num_angle.valueChanged.disconnect(self.update)
        self.chk_grid.toggled.disconnect(self.update)
        self.num_grid.valueChanged.disconnect(self.update)
        self.chk_reshape.toggled.disconnect(self.update)
        self.opt_new.toggled.disconnect(self.update)
        self.opt_replace.toggled.disconnect(self.update)

    def ok(self):
        # Draw figure if not already done
        # TODO: TAG: Functionality check
        if not hasattr(self.signal.signal, 'events') or \
                not self.gbo_preview.isChecked():
            self.update()
        angle = self.num_angle.value()
        reshape = self.chk_reshape.isChecked()
        self.plugin.record_code(
            r"<p>.rotate_signal({0}, reshape={1}, axes={2})".format(
                angle, reshape, self.axes))
        # Clean up event connections
        if self.new_out is not None:
            self.connect_update_plot(self.new_out.signal, disconnect=True)

    def close_new(self, value=False):
        if self.new_out is not None and not value:
            self.new_out.close()
            self.new_out = None
            self._connected_updates = False

    def set_preview(self, value):
        if not hasattr(self.signal.signal, 'events'):
            return
        if value:
            self.connect()
            self.update()
        else:
            self.disconnect()
            self.close_new()

    def _axes_in_nav(self):
        axm = self.signal.signal.axes_manager
        navidx = [axm._axes.index(ax) for ax in axm.navigation_axes]
        if self.axes[0] in navidx:
            return True
        return False

    def connect_update_plot(self, signal, disconnect=False):
        if self._connected_updates != disconnect:
            return  # Nothing to do, prevent double connections
        if self._axes_in_nav():
            f = signal._plot.navigator_plot.update
        else:
            f = signal._plot.signal_plot.update

        # TODO: TAG: Functionality check
        if hasattr(signal, 'events') and hasattr(
                signal.events, 'data_changed'):
            if disconnect:
                signal.events.data_changed.disconnect(f)
            else:
                signal.events.data_changed.connect(f, [])
        self._connected_updates = not disconnect

    def update(self):
        angle = self.num_angle.value()
        reshape = self.chk_reshape.isChecked()
        if self.opt_new.isChecked():
            if self.new_out is None:
                out = None
            else:
                out = self.new_out.signal
        elif self.opt_replace.isChecked():
            out = self.signal.signal
        else:
            return  # Indeterminate state, do nothing

        s = self.plugin.rotate_signal(angle, self.signal.signal, record=False,
                                      reshape=reshape, out=out, axes=self.axes)

        if out is None:
            s.metadata.General.title = self.signal.name + "[Rotated]"
            s.plot()
            self.connect_update_plot(s)
            if (self.gbo_preview.isChecked() and self.opt_new.isChecked() and
                                                         self.new_out is None):
                self.new_out = self.ui.lut_signalwrapper[s]
        else:
            s = out

        if self.chk_grid.isChecked() is True:
            pass    # TODO: Draw grid

    def create_controls(self):
        """
        Create UI controls.
        """
        vbox = QVBoxLayout()

        form = QFormLayout()
        self.num_angle = QDoubleSpinBox()
        self.num_angle.setValue(0.0)
        self.num_angle.setMinimum(-360)
        self.num_angle.setMaximum(360)
        form.addRow(tr("Angle:"), self.num_angle)
        vbox.addLayout(form)

        self.gbo_preview = QGroupBox(tr("Preview"))
        self.gbo_preview.setCheckable(True)
        self.gbo_preview.setChecked(False)
        gbo_vbox = QVBoxLayout()
        self.chk_grid = QCheckBox(tr("Grid"))
        self.chk_grid.setChecked(False)
        self.num_grid = QSpinBox()
        self.num_grid.setValue(4)
        self.num_grid.setMinimum(1)
        self.num_grid.setEnabled(False)
        self.chk_grid.toggled[bool].connect(self.num_grid.setEnabled)
        gbo_vbox.addWidget(self.chk_grid)
        gbo_vbox.addWidget(self.num_grid)
        self.gbo_preview.setLayout(gbo_vbox)
        vbox.addWidget(self.gbo_preview)

        self.gbo_preview.toggled[bool].connect(self.set_preview)

        self.gbo_output = QGroupBox(tr("Output"))
        self.opt_new = QRadioButton(tr("New signal"))
        self.opt_replace = QRadioButton(tr("In place"))
        self.opt_new.setChecked(True)
        gbo_vbox2 = QVBoxLayout()
        gbo_vbox2.addWidget(self.opt_new)
        gbo_vbox2.addWidget(self.opt_replace)
        self.gbo_output.setLayout(gbo_vbox2)
        vbox.addWidget(self.gbo_output)

        self.chk_reshape = QCheckBox(tr("Resize to fit"))
        self.chk_reshape.setChecked(False)
        vbox.addWidget(self.chk_reshape)

        self.btn_ok = QPushButton(tr("&OK"))
        self.btn_ok.setDefault(True)
        self.btn_ok.clicked.connect(self.accept)
        self.btn_cancel = QPushButton(tr("&Cancel"))
        self.btn_cancel.clicked.connect(self.reject)
        hbox = QHBoxLayout()
        hbox.addWidget(self.btn_ok)
        hbox.addWidget(self.btn_cancel)
        vbox.addLayout(hbox)

        vbox.addStretch(1)
        self.setLayout(vbox)



# -*- coding: utf-8 -*-
# Copyright 2014-2016 The HyperSpyUI developers
#
# This file is part of HyperSpyUI.
#
# HyperSpyUI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HyperSpyUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HyperSpyUI.  If not, see <http://www.gnu.org/licenses/>.

from hyperspyui.plugins.plugin import Plugin
from hyperspy.signal import BaseSignal
import numpy as np
from hyperspy.signals import Signal1D, Signal2D
from hyperspyui.tools import MultiSelectionTool
from hyperspyui.util import win2sig
from hyperspy.misc.rgb_tools import regular_array2rgbx

import matplotlib.cm as plt_cm

# TODO: Add dialog for manual editing of ROIs + Preview checkbox.


class Segmentation(Plugin):
    name = "Segmentation"

    def create_tools(self):
        self.tool = MultiSelectionTool()
        self.tool.name = 'Segmentation tool'
        self.tool.icon = 'segmentation.svg'
        self.tool.category = 'Image'
        self.tool.updated[BaseSignal, list].connect(self._on_update)
        self.tool.accepted[BaseSignal, list].connect(self.segment)
        self.tool.validator = self._tool_signal_validator
        self.add_tool(self.tool, self._select_image)
        self.map = {}
        self.ui.actions[self.tool.name].triggered.connect(
            lambda c=None: self.start())

    def _select_image(self, win, action):
        """Signal selection callback for actions that are only valid for
        selected Signals.
        """
        sw = win2sig(win, self.ui.signals, self.ui._plotting_signal)
        if sw is None or not sw.signal.axes_manager.signal_dimension == 2:
            action.setEnabled(False)
        else:
            action.setEnabled(True)

    def _tool_signal_validator(self, signal, axes):
        if not self.tool._default_validator(signal, axes):
            return False
        return signal in self.map

    def start(self, signal=None):
        if signal is None:
            signal = self.ui.get_selected_signal()
        data = signal()
        hist = signal._get_signal_signal(data).get_histogram(1000)
        hist.plot()

        s_out = Signal1D(self._make_gray(data))
        s_out.change_dtype('rgb8')
        s_out.plot()

        self.map[hist] = (signal, s_out)

    def _make_gray(self, data):
        data = data.astype(np.float) - np.nanmin(data)
        data /= np.nanmax(data)
        return (255 * plt_cm.gray(data)).astype('uint8')

    def segment(self, signal, rois):
        if signal is None:
            signal = self.ui.get_selected_signal()

        if signal in self.map:
            histogram = signal
            source, s_out = self.map[signal]
        else:
            found = False
            for h, (s, s_out) in self.map.items():
                if signal in (s, s_out):
                    found = True
                    histogram = h
                    source = s
                    break
            if not found:
                histogram = None
                s_out = None
                source = signal
        if histogram is not None:
            self.ui.lut_signalwrapper[histogram].close()
        if s_out is not None:
            self.ui.lut_signalwrapper[s_out].close()

        N = len(rois)
        if N <= 256:
            dtype = np.uint8
        elif N <= 2**16:
            dtype = np.uint16
        else:
            dtype = np.uint32
        src_data = source()
        data = np.zeros(src_data.shape, dtype)
        data[...] = np.nan
        for i, r in enumerate(rois):
            # Checks has to be inclusive to catch edges
            mask = (src_data <= r.right) & (src_data >= r.left)
            data[mask] = i + 1

        s_seg = Signal2D(data)
        s_seg.plot(cmap=plt_cm.jet)

        roi_str = '[' + ',\n'.join(['hs.roi.' + str(r) for r in rois]) + ']'
        self.record_code('segment_rois = ' + roi_str)
        self.record_code('<p>.segment(None, segment_rois)')

    def _on_update(self, histogram, rois):
        if histogram not in self.map:
            return
        source, s_out = self.map[histogram]
        N = len(rois)
        data = source()

        gray = self._make_gray(data)
        s_out.data = regular_array2rgbx(gray)
        for i in range(N):
            color = (255 * plt_cm.hsv([float(i) / max(N, 10)])).astype('uint8')
            color = regular_array2rgbx(color)
            r = rois[i]
            mask = (data < r.right) & (data >= r.left)
            s_out.data[mask] = color
        s_out.update_plot()



import os
import sys
from pathlib import Path
import subprocess
from hyperspy.io_plugins import io_plugins


_DESKTOP = \
    """[Desktop Entry]
Exec={} %F
Name=HyperSpy UI
Terminal=false
MimeType={}
Icon=application-x-hspy
Type=Application
X-MultipleArgs=true
Categories=Science;Physics;DataVisualization;

"""

_MIME = \
    """<?xml version="1.0"?>
<mime-info xmlns='http://www.freedesktop.org/standards/shared-mime-info'>
<mime-type type="application/x-{}">
<comment>{}</comment>
{}
</mime-type>
</mime-info>
"""

_PKGDIR = Path(__file__).resolve().parent
if not os.environ.get('XDG_DATA_HOME'):
    os.environ['XDG_DATA_HOME'] = os.path.expanduser('~/.local/share')
_MIMEPATH = os.path.join(os.environ["XDG_DATA_HOME"], "mime")
APPS_DIR = os.path.join(os.environ['XDG_DATA_HOME'], "applications/")


def create_mime_file(hspy_format, types=None, update_db=False):
    if not os.path.exists(_MIMEPATH):
        os.makedirs(_MIMEPATH)
    if not os.path.exists(os.path.join(_MIMEPATH, "packages")):
        os.makedirs(os.path.join(_MIMEPATH, "packages"))
    name = hspy_format.format_name
    if name == "HDF5":
        extensions = set(["hspy", "hdf5"])
        defext = "hspy"
    else:
        extensions = set([ext.lower() for ext in hspy_format.file_extensions])
        defext = hspy_format.file_extensions[hspy_format.default_extension]
    extstr = "\n".join(
        ["<glob pattern=\"*.{}\"/>".format(ext) for ext in extensions])
    if name == "HDF5":
        extstr += '<sub-class-of type="application/x-hdf"/>\n'
    elif defext in ("rpl", "msa", "dens"):
        extstr += '<sub-class-of type="text/plain"/>\n'
    mime = _MIME.format(defext, name, extstr)

    fpath = os.path.join(
        _MIMEPATH, "packages", "application-hyperspy-{}.xml".format(defext))
    print("Writing {}".format(fpath))
    with open(fpath, "w") as f:
        f.write(mime)
    if types:
        types.append("application/x-{}".format(defext))
    if update_db:
        print("Updating mime database")
        subprocess.run(['update-mime-database',  _MIMEPATH])


def get_hyperspyui_exec_path():
    p = subprocess.run(["which", "hyperspyui"], stdout=subprocess.PIPE)
    if p.stdout.decode('utf-8'):
        hspyui_exec_path = p.stdout.decode('utf-8').replace("\n", "")
    else:
        hspyui_exec_path = "hyperspyui"
    return hspyui_exec_path


def remove_desktop_file():
    fpath = os.path.join(APPS_DIR, "hyperspyui.desktop")
    if os.path.isfile(fpath):
        os.remove(fpath)
        print(fpath + " removed.")
    else:
        print("The file hyperspyui.desktop was not found at the default location %s" % fpath)
        print("Nothing done.")

def write_desktop_file(types=""):
    with open(os.path.join(APPS_DIR, "hyperspyui.desktop"), "w") as f:
        print("Writing hyperspyui.desktop to {}".format(APPS_DIR))
        f.write(_DESKTOP.format(get_hyperspyui_exec_path(), ";".join(types)))



def register_hspy_icon():
    for size in (16, 22, 32, 48, 64, 128, 256):
        print("Registering icon size {}".format(size))
        subprocess.run(["xdg-icon-resource", "install",  "--context",
                        "mimetypes",
                        "--size", str(size),
                        "{}/images/icon/hyperspy{}.png".format(_PKGDIR, size),
                        "application-x-hspy"])


def run_desktop_integration_linux(args,):
    if args.remove:
        remove_desktop_file()
    else:
        exclude_formats=["netCDF", "Signal2D", "Protochips", "TIFF"]
        types = ["image/tiff"]
        for hspy_format in io_plugins:
            if hspy_format.format_name not in exclude_formats:
                create_mime_file(hspy_format=hspy_format, types=types)
        print("Updating mime database")
        subprocess.run(['update-mime-database',  _MIMEPATH])
        register_hspy_icon()
        write_desktop_file(types=types)
        print("Updating desktop database")
        subprocess.run(['update-desktop-database', APPS_DIR])
